// 
// $Id$
//
// Implementation of eZFile class
//
// Jan Borsodi <amos@ez.no>
// Created on: <04-Dec-1999 23:28:13 amos>
//


#include "ezfile.hpp"
#include <qstring.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qdir.h>

/*!
  \class eZFile ezfile.hpp
  \brief

*/

/*!
  Default constructor
*/

eZFile::eZFile()
{
}

/*!
  Destroys the object
*/

eZFile::~eZFile()
{
}

eZFile::CopyType eZFile::copyFile( const QString &srcdir, const QString &srcfile,
                                   const QString &dstdir, const QString &dstfile,
                                   CopyType oldtype )
{
    QString src = srcdir + "/" + srcfile;
    QString dst = dstdir + "/" + dstfile;
    QFileInfo dstdirinf( dstdir );
    if ( !dstdirinf.exists() )
    {
        int num = QMessageBox::warning( 0,
                                        "Destination directory doesn't exists",
                                        QString( "Destination directory \"%1\" doesn't exists,\ndo you wish to create it?" )
                                        .arg( dstdir ),
                                        "&Yes", "&No",
                                        0, 1 );
        if ( num == 1 )
            return Abort;
        QDir d;
        bool again = true;
        while ( again )
        {
            if ( eZFile::createDirectory( dstdir ) )
            {
                again = false;
            }
            else
            {
                int num = QMessageBox::warning( 0,
                                                "Couldn't create destination",
                                                QString( "Couldn't create destination directory \"%1\"" )
                                                .arg( dstdir ),
                                                "&Retry", "&Abort",
                                                0, 1 );
                if ( num == 1 )
                    return Abort;
            }
        }
    }
    QFileInfo srcinf( src );
    bool ok = false;
    CopyType t = Ok;
    while ( !ok )
    {
        if ( srcinf.exists() )
        {
            QFileInfo dstinf( dst );
            bool write = true;
            if ( dstinf.exists() )
            {
                if ( oldtype == SkipAll )
                    return oldtype;
                if ( oldtype == OverwriteAll )
                {
                    t = oldtype;
                    write = true;
                }
                else
                {
                    int num = QMessageBox::warning( 0,
                                                    "Destination alreay exists",
                                                    QString( "Destination file \"%1\"already exists,\n are you sure you want to overwrite it?" )
                                                    .arg( dst ),
                                                    "&Overwrite", "Overwrite &All", "&Skip",
                                                    0, 2 );
                    if ( num == 2 )
                        write = false;
                    if ( num == 0 )
                        t = Overwrite;
                    else if ( num == 1 )
                        t = OverwriteAll;
                    else
                        t = Skip;
                }
            }
            if ( write )
            {
                QFile srcf( src ), dstf( dst );
                if ( srcf.open( IO_ReadOnly ) )
                {
                    bool retry = true;
                    while ( retry )
                    {
                        retry = false;
                        if ( dstf.open( IO_WriteOnly ) )
                        {
                            char buf[10*1024];
                            unsigned int size = srcf.size();
                            while ( size > 0 )
                            {
                                int read = srcf.readBlock( buf, 10*1024 );
                                retry = true;
                                while( retry )
                                {
                                    retry = false;
                                    int wrote = dstf.writeBlock( buf, read );
                                    if ( read != wrote )
                                    {
                                        if ( QMessageBox::critical( 0,
                                                                    "Error, couldn't write to destination",
                                                                    QString( "Error, couldn't write to destination file\n\"%1\"\nprobably out of space" )
                                                                    .arg( dst ),
                                                                    "&Retry", "&Skip", QString::null,
                                                                    0, 1 ) == 1 )
                                            return Skip;
                                        else
                                            retry = true;
                                    }
                                }
                                size -= read;
                            }
                            ok = true;
                        }
                        else
                        {
                            if ( QMessageBox::critical( 0,
                                                        "Error, couldn't open destination",
                                                        QString( "Error, couldn't open destination file\n\"%1\" for writing\n unable to copy file" )
                                                        .arg( dst ),
                                                        "&Retry", "&Skip", QString::null,
                                                        0, 1 ) == 1 )
                                return Skip;
                            else
                                retry = true;
                        }
                    }
                }
                else
                {
                    if ( QMessageBox::critical( 0,
                                                "Error, couldn't open source",
                                                QString( "Error, couldn't open source file\n\"%1\" for reading\n unable to copy file" )
                                                .arg( src ),
                                                "&Retry", "&Skip", QString::null,
                                                0, 1 ) == 1 )
                        return Skip;
                }
            }
            return t;
        }
        else
        {
            if ( QMessageBox::critical( 0,
                                        "Error, source doesn't exists",
                                        QString( "Error, source file\n\"%1\" doesn't exists\n unable to copy file" )
                                        .arg( src ),
                                        "&Retry", "&Skip", QString::null,
                                        0, 1 ) == 1 )
                return Skip;
        }
    }
    return t;
}

bool eZFile::createDirectory( const QString &dir )
{
    QStringList strs;
    int index = 0;
    if ( dir.at( 0 ) == QChar( '/' ) )
        index++;
    int oldindex = index;
    while ( ( index = dir.find( '/', index ) ) != -1 )
    {
        QString str = dir.mid( 0, index );
        strs.append( str );
        index++;
        oldindex = index;
    }
    if ( oldindex != (int)dir.length() )
    {
        strs.append( dir.mid( 0 ) );
    }
    QFileInfo inf;
    QDir d;
    for ( QStringList::Iterator it = strs.begin(); it != strs.end(); ++it )
    {
        inf.setFile( *it );
        if ( inf.exists() )
        {
            if ( !inf.isDir() )
                return false;
        }
        else
        {
            if ( !d.mkdir( *it ) )
                return false;
        }
    }
    return true;
}
