// 
// $Id$
//
// Implementation of eZIconSelector class
//
// Jan Borsodi <amos@ez.no>
// Created on: <04-Dec-1999 21:45:59 amos>
//


#include "eziconselector.hpp"
#include <qwidget.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include "eziconset.hpp"
#include <qradiobutton.h>
#include <qlabel.h>
#include <qstring.h>
#include <qdir.h>

/*!
  \class eZIconSelector eziconselector.hpp
  \brief

*/

/*!
  Default constructor
*/

eZIconSelector::eZIconSelector( bool superuser, QWidget *parent, const char *name )
    : QWidget( parent, name )
{
    QVBoxLayout *lay = new QVBoxLayout( this, 3, 5, "toplayout" );
    {
        QGroupBox *box = new QGroupBox( "Iconset", this, "icon box" );
        lay->addWidget( box, 1 );
        {
            QVBoxLayout *mlay = new QVBoxLayout( box, 8, 5, "program layout" );
            {
                mlay->addSpacing( box->fontMetrics().lineSpacing() );

                QLabel *lab = new QLabel( "The iconset selected will be used on the toolbar in QtVu.\n"
                                          "Install directory is ~/.QtVu/icons", box );
                mlay->addWidget( lab, 0 );

                mlay->addSpacing( 20 );

                Pressed = DefBut = new QRadioButton( "Default", box );
                Busy = false;
                DefBut->setChecked( true );
                connect( DefBut, SIGNAL( toggled( bool ) ),
                         this, SLOT( defChecked( bool ) ) );
                mlay->addWidget( DefBut, 0 );

                eZIconSet *defset = new eZIconSet( "icons-default", box );
                mlay->addWidget( defset, 0 );

                NewBut = new QRadioButton( "Fancy", box );
                connect( NewBut, SIGNAL( toggled( bool ) ),
                         this, SLOT( newChecked( bool ) ) );
                mlay->addWidget( NewBut, 0 );

                eZIconSet *newset = new eZIconSet( "icons-new", box );
                mlay->addWidget( newset, 0 );

                mlay->addStretch( 1 );
            }
        }

        QGroupBox *box2 = new QGroupBox( "Icon location", this, "icon box" );
        lay->addWidget( box2, 1 );
        {
            QVBoxLayout *mlay = new QVBoxLayout( box2, 8, 5, "program layout" );
            {
                mlay->addSpacing( box2->fontMetrics().lineSpacing() );

                QLabel *lab = new QLabel( "Select location for icons.", box2 );
                mlay->addWidget( lab, 0 );

                mlay->addSpacing( 20 );

                Pressed2 = HomeBut = new QRadioButton( QDir::homeDirPath() + "/.QtVu/icons", box2 );
                Busy2 = false;
                HomeBut->setChecked( true );
                connect( HomeBut, SIGNAL( toggled( bool ) ),
                         this, SLOT( homeChecked( bool ) ) );
                mlay->addWidget( HomeBut, 0 );

                ShareBut = new QRadioButton( "/usr/share/icons", box2 );
                ShareBut->setEnabled( superuser );
                connect( ShareBut, SIGNAL( toggled( bool ) ),
                         this, SLOT( shareChecked( bool ) ) );
                mlay->addWidget( ShareBut, 0 );

                mlay->addStretch( 1 );
            }
        }
    }
}

/*!
  Destroys the object
*/

eZIconSelector::~eZIconSelector()
{
}

void eZIconSelector::defChecked( bool b )
{
    if ( Busy )
        return;
    Busy = true;
    if ( b )
    {
        NewBut->setChecked( false );
        Pressed = DefBut;
    }
    else
    {
        DefBut->setChecked( true );
    }
    Busy = false;
}

void eZIconSelector::newChecked( bool b )
{
    if ( Busy )
        return;
    Busy = true;
    if ( b )
    {
        DefBut->setChecked( false );
        Pressed = NewBut;
    }
    else
    {
        NewBut->setChecked( true );
    }
    Busy = false;
}

void eZIconSelector::homeChecked( bool b )
{
    if ( Busy2 )
        return;
    Busy2 = true;
    if ( b )
    {
        ShareBut->setChecked( false );
        Pressed2 = HomeBut;
    }
    else
    {
        HomeBut->setChecked( true );
    }
    Busy2 = false;
}

void eZIconSelector::shareChecked( bool b )
{
    if ( Busy2 )
        return;
    Busy2 = true;
    if ( b )
    {
        HomeBut->setChecked( false );
        Pressed2 = ShareBut;
    }
    else
    {
        ShareBut->setChecked( true );
    }
    Busy2 = false;
}

QString eZIconSelector::iconDirectory() const
{
    if ( Pressed == NewBut )
        return QString( "icons-new" );
    else
        return QString( "icons-default" );
}

QString eZIconSelector::iconDestDirectory() const
{
    if ( Pressed2 == ShareBut )
        return QString( "/usr/share/icons" );
    else
        return QString( QDir::homeDirPath() + "/.QtVu/icons" );
}
