// 
// $Id: ezlocationselector.cpp,v 1.1.1.1 1999/12/04 20:30:05 amos Exp $
//
// Implementation of eZLocationSelector class
//
// Jan Borsodi <amos@ez.no>
// Created on: <19-Aug-1999 23:58:36 amos>
//


#include "ezlocationselector.hpp"
#include <qwidget.h>
#include <qlayout.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qlistbox.h>
#include <qlabel.h>
#include <qdir.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qvbox.h>
#include <qstring.h>
#include <qfiledialog.h>
#include <qcheckbox.h>
#include <qfileinfo.h>
#include <qfile.h>

/*!
  \class eZLocationSelector ezlocationselector.hpp
  \brief

*/

/*!
  Default constructor
*/

eZLocationSelector::eZLocationSelector( const QString &progname, QWidget *parent, const char *name )
    : QWidget(  parent, name ),
    bBin( true ), bInclude( false ), bLib( false ), bDoc( false ),
    bAddBin( true ), bAddInclude( true ), bAddLib( true ), bAddDoc( true ),
    DirOpt( 0 ), DirUsr( 0 ), DirUsrLib( 0 ), DirUsrLocal( 0 ), DirUsrLocalLib( 0 ), DirHome( 0 )
{
    Browsed = false;
    LastIndex = 1;
    setCaption( QString( "eZInstall - Installing %1" ).arg( progname ) );

    CharLen = (int)(('z'-'a') + 1 + ('Z' - 'A') + 1 + ('9'-'0') + 1);
    Chars = new char[CharLen];
    int index = 0;
    for ( char c = 'a'; c <= 'z'; c++ )
        Chars[index++] = c;
    for ( char c = 'A'; c <= 'Z'; c++ )
        Chars[index++] = c;
    for ( char c = '0'; c <= '9'; c++ )
        Chars[index++] = c;

    QHBoxLayout *main = new QHBoxLayout( this, 3, 5, "toplayout" );
    {
        QVBoxLayout *lay = new QVBoxLayout();
        main->addLayout( lay, 3 );
        {
            QGroupBox *box = new QGroupBox( "Program location", this, "program box" );
            lay->addWidget( box, 1 );
            {
                QVBoxLayout *mlay = new QVBoxLayout( box, 8, 5, "program layout" );
                {
                    mlay->addSpacing( box->fontMetrics().lineSpacing() );

                    QLabel *l = new QLabel( "Select one of the standard installation directories, "
                                            "or choose Custom to a select directory of your choice.", box, "toptext" );
                    l->setAlignment( AlignTop | AlignLeft | WordBreak );
                    mlay->addWidget( l, 0 );

                    mlay->addSpacing( 20 );

                    Check = new QCheckBox( "Install in a sub &directory called", box );
                    Check->setChecked( false );
                    connect( Check, SIGNAL( toggled( bool ) ),
                             this, SLOT( dirToggled( bool ) ) );
                    mlay->addWidget( Check, 0 );

                    SubLine = new QLineEdit( progname, box );
                    SubLine->setEnabled( false );
                    mlay->addWidget( SubLine, 0 );
                    connect( SubLine, SIGNAL( textChanged( const QString & ) ),
                             this, SLOT( setDestination() ) );

                    BinCheck = new QCheckBox( "Install in a \"&bin\" directory", box );
                    BinCheck->setChecked( bAddBin );
                    connect( BinCheck, SIGNAL( toggled( bool ) ),
                             this, SLOT( binToggled( bool ) ) );
                    mlay->addWidget( BinCheck, 0 );

                    QHBoxLayout *hlay = new QHBoxLayout( -1, "program hlayout");
                    mlay->addLayout( hlay, 1 );
                    {
                        LBox = new QListBox( box );

                        LBox->insertItem( "Custom.." );
                        Strings.append( "Install the program in\na place of your choice." );

                        if ( canWrite( "/opt" ) )
                        {
                            DirOpt = new QListBoxText( LBox, "/opt" );
                            LBox->insertItem( DirOpt );
                            Strings.append( "Commonly used for larger programs." );
                        }

                        if ( canWrite( "/usr" ) )
                        {
                            DirUsr = new QListBoxText( LBox, "/usr" );
                            LBox->insertItem( DirUsr );
                            Strings.append( "Standard install path for root user." );
                        }

                        if ( canWrite( "/usr/lib" ) )
                        {
                            DirUsrLib = new QListBoxText( LBox, "/usr/lib" );
                            LBox->insertItem( DirUsrLib );
                            Strings.append( "Standard install path for libraries." );
                        }

                        if ( canWrite( "/usr/local" ) )
                        {
                            DirUsrLocal = new QListBoxText( LBox, "/usr/local" );
                            LBox->insertItem( DirUsrLocal );
                            Strings.append( "Standard install path for common user." );
                        }

                        if ( canWrite( "/usr/local/lib" ) )
                        {
                            DirUsrLocalLib = new QListBoxText( LBox, "/usr/local/lib" );
                            LBox->insertItem( DirUsrLocalLib );
                            Strings.append( "Standard install path for libraries for common user." );
                        }

                        if ( canWrite( QDir::homeDirPath() ) )
                        {
                            DirHome = new QListBoxText( LBox, QDir::homeDirPath() );
                            LBox->insertItem( DirHome );
                            Strings.append( "Users home directory." );
                        }

                        LBox->setMinimumSize( 90, 120 );

                        connect( LBox, SIGNAL( highlighted( int ) ),
                                 this, SLOT( listSelect( int ) ) );

                        hlay->addWidget( LBox, 1 );

                        InfoLabel = new QLabel( "Install the program in\na place of your choice.", box );
                        InfoLabel->setAlignment( AlignLeft | AlignTop | WordBreak );
                        hlay->addWidget( InfoLabel, 1 );
                    }
                }
            }

            GBox = new QGroupBox( "Custom location", this );
            GBox->setEnabled( false );
            lay->addWidget( GBox );
            {
                QVBoxLayout *vlay = new QVBoxLayout( GBox, 8, 5, "custom vlayout" );
                {
                    vlay->addSpacing( GBox->fontMetrics().lineSpacing() );

                    QHBoxLayout *hlay = new QHBoxLayout( -1, "custom layout" );
                    vlay->addLayout( hlay, 1 );
                    {
                        DLine = new QLineEdit( GBox );
                        DLine->setEnabled( false );
                        connect( DLine, SIGNAL( textChanged( const QString & ) ),
                                 this, SLOT( setDestination() ) );
                        hlay->addWidget( DLine, 1 );

                        Browse = new QPushButton( "Bro&wse", GBox );
                        Browse->setEnabled( false );

                        connect( Browse, SIGNAL( clicked() ),
                                 this, SLOT( browseClicked() ) );

                        hlay->addWidget( Browse, 0 );
                    }
                }
            }

            QVBoxLayout *inflay = new QVBoxLayout();
            lay->addLayout( inflay, 0 );
            {
                InfBox = new QGroupBox( "Destinations", this, "destination box" );
                inflay->addWidget( InfBox, 0 );
                {
                    QVBoxLayout *mlay = new QVBoxLayout( InfBox, 8, 5 );
                    {
                        mlay->addSpacing( InfBox->fontMetrics().lineSpacing() );

                        DestLabel = new QLabel( InfBox );
                        mlay->addWidget( DestLabel, 0 );
                    }
                }
            }
        }
    }

    if ( DirUsrLocal != 0 )
        LBox->setCurrentItem( DirUsrLocal );
    else if ( DirUsr != 0 )
        LBox->setCurrentItem( DirUsr );
    else if ( DirUsrLocalLib != 0 )
        LBox->setCurrentItem( DirUsrLocalLib );
    else if ( DirUsrLib != 0 )
        LBox->setCurrentItem( DirUsrLib );
    else if ( DirOpt != 0 )
        LBox->setCurrentItem( DirOpt );
    else if ( DirHome != 0 )
        LBox->setCurrentItem( DirHome );
    else
        LBox->setCurrentItem( 0 );
    setDestination();
    resize( 500, 400 );
}

/*!
  Destroys the object
*/

eZLocationSelector::~eZLocationSelector()
{
    delete []Chars;
}

void eZLocationSelector::listSelect( int i )
{
    if ( i == 0 )
    {
        GBox->setEnabled( true );
        DLine->setEnabled( true );
        Browse->setEnabled( true );
        if ( !Browsed )
        {
            Browse->animateClick();
        }
        else
            LastIndex = i;
    }
    else
    {
        LastIndex = i;
        GBox->setEnabled( false );
        DLine->setEnabled( false );
        Browse->setEnabled( false );
    }
    InfoLabel->setText( Strings[i] );
    setDestination();
}

void eZLocationSelector::browseClicked()
{
    QString str = QFileDialog::getExistingDirectory( DLine->text(), this );
    if ( !str.isNull() )
    {
        Browsed = true;
        DLine->setText( str );
    }
    else if ( !Browsed )
    {
        if ( LastIndex >= (int)LBox->count() )
        {
            Browsed = true;
            DLine->setText( QDir::currentDirPath() );
            LBox->setCurrentItem( 0 );
        }
        else
            LBox->setCurrentItem( LastIndex );
    }
    setDestination();
}

void eZLocationSelector::dirToggled( bool b )
{
    if ( b )
    {
        SubLine->setEnabled( true );
    }
    else
    {
        SubLine->setEnabled( false );
    }
    setDestination();
}

void eZLocationSelector::setDestination()
{
    QString str;
    if ( bBin )
        str += ( "Binary path: \"%1\"\n" );
    if ( bInclude )
        str += ("Include path: \"%2\"\n" );
    if ( bLib )
        str += ( "Library path: \"%3\"\n" );
    if ( bDoc )
        str += ( "Document path: \"%4\"\n" );

    QString path;
    if ( LBox->currentItem() == 0 )
        path = DLine->text();
    else
        path = LBox->item( LBox->currentItem() )->text();
    if ( Check->isChecked() )
        if ( !SubLine->text().isEmpty() )
            path += "/" + SubLine->text();
    path = QDir::cleanDirPath( path );
    QString bin;
    QString inc;
    QString lib;
    QString doc;
    QString txt;
    bin = path;
    if ( bAddBin )
        bin += "/bin";
    inc = path;
    if ( bAddInclude )
        inc += "/include";
    lib = path;
    if ( bAddLib )
        lib += "/lib";
    doc = path;
    if ( bAddDoc )
        doc += "/doc";
    txt = str;
    if ( bBin )
        txt = txt.arg( bin );
    if ( bInclude )
        txt = txt.arg( inc );
    if ( bLib )
        txt = txt.arg( lib );
    if ( bDoc )
        txt = txt.arg( doc );
    DestLabel->setText( txt );
}

QString eZLocationSelector::installPath() const
{
    QString path;
    if ( LBox->currentItem() == 0 )
        path = DLine->text();
    else
        path = LBox->item( LBox->currentItem() )->text();
    if ( Check->isChecked() )
        if ( !SubLine->text().isEmpty() )
            path += "/" + SubLine->text();
    return path;
}

QString eZLocationSelector::binaryPath() const
{
    if ( bBin )
    {
        if ( bAddBin )
            return QDir::cleanDirPath( installPath() ) + "/bin";
        else
            return QDir::cleanDirPath( installPath() );
    }
    else
        return QString::null;
}

bool eZLocationSelector::canWrite( const QString &dir )
{
    QDir d( dir );
    QString realdir = d.absPath();
    int num = 0;
    bool fin = false;
    QString tmp;
    while ( !fin )
    {
        tmp = tmpName( num );
        QFileInfo inf( realdir + "/" + tmp );
        if ( !inf.exists() )
        {
            fin = true;
        }
        else
            num++;
    }
    QFile f( realdir + "/" + tmp );
    if ( f.open( IO_WriteOnly ) )
    {
        f.close();
        f.remove();
        return true;
    }
    return false;
}

QString eZLocationSelector::tmpName( int num )
{
    unsigned int fib[16];
    for ( int i = 0; i < 16; i++ )
    {
        if ( i >= 2 )
            fib[i] = fib[i-1] + 2*fib[i-2];
        else if ( i == 1 )
            fib[i] = fib[i-1];
        else
            fib[i] = 1;
    }
    char tmp[16+1];
    unsigned int index = num + fib[num&15];
    for ( int i = 0; i < 16; i++ )
    {
        index += fib[i];
        tmp[i] = Chars[(index)%CharLen];
        fib[i] += fib[(i + index)&15];
    }
    tmp[16] = 0;
    return QString( tmp );
}

void eZLocationSelector::binToggled( bool b )
{
    bAddBin = b;
    setDestination();
}
