// 
// $Id: main.cpp,v 1.1.1.1 1999/12/04 20:30:05 amos Exp $
//
// Main body implementation
//
// Jan Borsodi <amos@ez.no>
// Created on: <19-Aug-1999 22:26:55 amos>
//

#include "ezlocationselector.hpp"
#include "eziconselector.hpp"
#include <qapplication.h>
#include <qwizard.h>
#include <qstring.h>
#include "ezfile.hpp"
#include <qdir.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <qmessagebox.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <qfile.h>
#include <qtextstream.h>

void help()
{
    printf( "QtVu Installer 0.1\n"
            "Usage: install [options]\n"
            "Options:"
            "  -h, --help    Displays this help\n"
            "  -i, --icons   Install only icons\n"
            "  -p, --program Install only program\n\n"
            "No options means install program and icons\n" );
    exit( 0 );
}

int main( int argc, char **argv )
{
    int uid = getuid();
    QApplication app( argc, argv );

    bool ProgramInstall = true;
    bool IconInstall = true;
    if ( argc > 1 )
    {
        for ( int i = 1; i < argc; i++ )
        {
            if ( strcmp( argv[i], "-h" ) == 0 )
                help();
            else if ( strcmp( argv[i], "--help" ) == 0 )
                help();
            else if ( strcmp( argv[i], "-i" ) == 0 )
            {
                ProgramInstall = false;
                IconInstall = true;
            }
            else if ( strcmp( argv[i], "--icons" ) == 0 )
            {
                ProgramInstall = false;
                IconInstall = true;
            }
            else if ( strcmp( argv[i], "-p" ) == 0 )
            {
                ProgramInstall = true;
                IconInstall = false;
            }
            else if ( strcmp( argv[i], "--program" ) == 0 )
            {
                ProgramInstall = true;
                IconInstall = false;
            }
        }
    }

    if ( uid != 0 )
    {
        QMessageBox::warning( 0, "Not running as SuperUser",
                              "You are not running as super user,\n"
                              "this means that there probably are\n"
                              "a lot of directories you cannot install to.",
                              0 );
    }
    QWizard w( 0, 0, true );
    w.setCaption( "eZInstall - Installing QtVu 0.3.21beta" );
    eZLocationSelector *loc;
    eZIconSelector *icon;
    if ( ProgramInstall )
    {
        loc = new eZLocationSelector( "QtVu", &w );
        w.addPage( loc, "Program installation" );
    }
    if ( IconInstall )
    {
        icon = new eZIconSelector( uid == 0, &w );
        w.addPage( icon, "Icon installation");
    }
    if ( ProgramInstall )
    {
        if ( !IconInstall )
            w.setFinishEnabled( loc, true );
        w.setFinishEnabled( icon, true );
    }
    else
        w.setFinishEnabled( icon, true );
    if ( w.exec() )
    {
        eZFile::CopyType t = eZFile::Ok;
        if ( ProgramInstall )
        {
            t = eZFile::copyFile( ".", "qtvu",
                                  loc->binaryPath(), "qtvu" );
            if ( t == eZFile::Ok )
                if ( chmod( QString( loc->binaryPath() + "/qtvu" ).ascii(), S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH ) != 0 )
                    t = eZFile::Skip;
        }
        if ( IconInstall )
        {
            t = eZFile::copyFile( icon->iconDirectory(), "back.xpm",
                                  icon->iconDestDirectory(), "back.xpm", t );
            t = eZFile::copyFile( icon->iconDirectory(), "fileopen.xpm",
                                  icon->iconDestDirectory(), "fileopen.xpm", t );
            t = eZFile::copyFile( icon->iconDirectory(), "forward.xpm",
                                  icon->iconDestDirectory(), "forward.xpm", t );
            t = eZFile::copyFile( icon->iconDirectory(), "slide.xpm",
                                  icon->iconDestDirectory(), "slide.xpm", t );
            t = eZFile::copyFile( icon->iconDirectory(), "zoomin.xpm",
                                  icon->iconDestDirectory(), "zoomin.xpm", t );
            t = eZFile::copyFile( icon->iconDirectory(), "zoomout.xpm",
                                  icon->iconDestDirectory(), "zoomout.xpm", t );
        }

        if ( t != eZFile::Abort && ProgramInstall )
        {
            QString insttxt( "QtVu is now installed in %1\n" );
            insttxt = insttxt.arg( loc->binaryPath() );
            if ( IconInstall )
                insttxt += QString( "Icons installed in %2\n" ).arg( icon->iconDestDirectory() );
            insttxt += "\nDo you wish to start QtVu?";
            int start = QMessageBox::information( 0, "QtVu installed",
                                                  insttxt,
                                                  "&Yes", "&No", QString::null,
                                                  0, 1 );
            if ( start == 0 )
            {
                execl( QString( loc->binaryPath() + "/qtvu" ).ascii(), 0 );
            }
        }
    }

    return 0;
}
