// qaconfig.cpp
//
// Jan Borsodi <amos@abn.hibu.no>
// Created on: <25-Apr-1999 23:19:29 amos>
// Last change: Time-stamp: <03-Dec-1999 02:10:01 amos>
//
//

#include <iostream.h>
#include "qaconfig.hpp"

#include <qdir.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qstring.h>
#include <qsize.h>
#include <qregexp.h>
#include <qlist.h>
#include <qtextstream.h>
#include <iostream>

QaConfig *QaConfig::_instance = 0;

/*!
  \class QaConfig qaconfig.hpp
  \brief

*/

/*!
  Default constructor
*/

QaConfig::QaConfig()
{
    Name = QDir::homeDirPath() + "/.QtVu/config";
    AbsName = "/usr/lib/qtvu/config";
    HotName = QDir::homeDirPath() + "/.QtVu/hotlist";
    AbsHotName = "/usr/lib/qtvu/hotlist";
    WindowSize = QSize( 400, 400 );
    LogSize = QSize( -1, -1 );
    ScrollDelay = 10;
    PixelsToScroll = 3;
    bHotDir = false;
    bSmoothGUI = false;
    bFullScreen = false;
    bFastDir = false;
    SplitterSizes.append( 100 );
    SplitterSizes.append( 100 );
    DividerSizes.append( 300 );
    DividerSizes.append( 300 );

    ParseList.append( new QaConfigParserSize );
    ParseList.append( new QaConfigParserLogSize );
    ParseList.append( new QaConfigParserScroll );
    ParseList.append( new QaConfigParserDirHot );
    ParseList.append( new QaConfigParserSmooth );
    ParseList.append( new QaConfigParserSplitter );
    ParseList.append( new QaConfigParserDivider );
    ParseList.append( new QaConfigParserFast );
    ParseList.append( new QaConfigParserFull );

    QFileInfo finf( QDir::homeDirPath() + "/.qtvurc" );
    if ( finf.exists() )
    {
        int ret =  QMessageBox::information( 0,
                                             "File Information",
                                             "Found an obsolete .qtvurc in your home directory\n"
                                             "Do you wish to remove it?",
                                             "&Ok", "&Cancel", 0,
                                             0, 1);
        if ( ret == 0 )
        {
            QDir d;
            bool fin = false;
            while( !fin )
            {
                if ( d.remove( QDir::homeDirPath() + "/.qtvurc" ) )
                {
                    fin = true;
                }
                else
                {
                    if ( QMessageBox::warning( 0,
                                               "File Information",
                                               "Failed to remove the .qtvurc file",
                                               QMessageBox::Retry | QMessageBox::Default,
                                               QMessageBox::Cancel | QMessageBox::Escape) == 1 )
                        fin = true;
                }
            }
        }
    }
    QString configname;
    finf.setFile( AbsName );
    if ( finf.exists() )
        configname = AbsName;
    else
        configname = Name;
    QFile config( configname );
    if ( !config.open( IO_ReadOnly ) )
    {
        qDebug( "Didn't find a config file, creating...." );
        save();
    }
    else
    {
        qDebug( "QaConfig reading in config\n" );
        char buf[200];
        config.readLine( buf, 200 );
        if ( strcmp( buf, "# QtVu Config File\n" ) == 0 )
        {
            while ( !config.atEnd() )
            {
                config.readLine( buf, 200 );
                bool err = true;
                for ( QListIterator<QaConfigParser> i( ParseList ); i.current(); ++i )
                {
                    if ( i.current()->parseString( QString( buf ), this ) )
                    {
                        err = false;
                        break;
                    }
                }
            }
        }
        else
        {
            qDebug( "Error reading config file\n" );
        }
    }
    parseHotList();
}

/*!
  Destroys the object
*/

QaConfig::~QaConfig()
{
    for ( QListIterator<QaConfigParser> i( ParseList ); i.current(); ++i )
    {
        delete i.current();
    }
    _instance = 0;
}

void QaConfig::save()
{
    qDebug( "Saving QtVu configuration...." );
    QFileInfo fi( QDir::homeDirPath() + "/.QtVu/" );
    if ( !fi.exists() )
    {
        QDir d;
        d.mkdir( QDir::homeDirPath() + "/.QtVu/" );
    }
    QFile conf( name() );
    if ( conf.open( IO_WriteOnly ) )
    {
        QTextStream s( &conf );

        s << "# QtVu Config File\n\n";

        for ( QListIterator<QaConfigParser> i( ParseList ); i.current(); ++i )
        {
            i.current()->save( s, this );
        }

        conf.close();

        conf.setName( hotName() );
        conf.open( IO_WriteOnly );
        s.setDevice( &conf );
        s << "# QtVu Hotlist\n\n";
        QStringList::ConstIterator i2 = nameHotList().begin();
        for ( QStringList::ConstIterator i = hotList().begin(); i != hotList().end() && i2 != nameHotList().end(); ++i, ++i2 )
        {
            s << (*i2) << " " << (*i) << endl;
        }
        conf.close();
    }
}


void QaConfig::parseHotList()
{
    QFile config( hotName() );
    if ( config.open( IO_ReadOnly ) )
    {
        qDebug( "QaConfig reading in hotlist" );
        QTextStream s( &config );
        QString buf;
        buf = s.readLine();
        if ( buf == "# QtVu Hotlist" )
        {
            while ( !config.atEnd() )
            {
                QString name, dir;
                s >> name;
                s >> dir;
                if ( name != "" && dir != "" )
                {
                    addToNameHotList( name );
                    addToHotList( dir );
                }
            }
        }
        else
        {
            qDebug( "Error reading hotlist" );
        }
    }
}

QString QaConfig::boolToAscii( bool b ) const
{
    if ( b )
        return "Yes";
    else
        return "No";
}

QaConfig *QaConfig::instance()
{
    if ( _instance == 0 )
        _instance = new QaConfig();
    return _instance;
}

void QaConfig::setLogSize( const QSize &s )
{
    LogSize = s;
}

const QSize &QaConfig::logSize() const
{
    return LogSize;
}

void QaConfig::setWindowSize( const QSize &s )
{
    WindowSize = s;
}

void QaConfig::setWindowWidth( const int s )
{
    WindowSize.setWidth( s );
}

void QaConfig::setWindowHeight( const int s )
{
    WindowSize.setHeight( s );
}

void QaConfig::setHotList( const QStringList &l )
{
    HotList = l;
}

void QaConfig::addToHotList( const QString &s )
{
    HotList.append( s );
}

void QaConfig::setNameHotList( const QStringList &l )
{
    NameHotList = l;
}

void QaConfig::addToNameHotList( const QString &s )
{
    NameHotList.append( s );
}

void QaConfig::setHotDir( bool b )
{
    bHotDir = b;
}

void QaConfig::setFullScreen( bool b )
{
    bFullScreen = b;
}

void QaConfig::setSmoothGUI( bool b )
{
    bSmoothGUI = b;
}

void QaConfig::setFastDir( bool b )
{
    bFastDir = b;
}

void QaConfig::setSplitterSizes( const QValueList<int> &v )
{
    SplitterSizes = v;
}

void QaConfig::setDividerSizes( const QValueList<int> &v )
{
    DividerSizes = v;
}

const QValueList<int> &QaConfig::splitterSizes() const
{
    return SplitterSizes;
}

const QValueList<int> &QaConfig::dividerSizes() const
{
    return DividerSizes;
}

bool QaConfig::hotDir() const
{
    return bHotDir;
}

bool QaConfig::fastDir() const
{
    return bFastDir;
}

bool QaConfig::smoothGUI() const
{
    return bSmoothGUI;
}

bool QaConfig::fullScreen() const
{
    return bFullScreen;
}

const QSize &QaConfig::windowSize() const
{
    return WindowSize;
}

const QString &QaConfig::name() const
{
    return Name;
}

const QString &QaConfig::hotName() const
{
    return HotName;
}

const QString &QaConfig::absHotName() const
{
    return AbsHotName;
}

const QStringList &QaConfig::hotList() const
{
    return HotList;
}

const QStringList &QaConfig::nameHotList() const
{
    return NameHotList;
}

int QaConfig::scrollDelay() const
{
    return ScrollDelay;
}

int QaConfig::scrollPixels() const
{
    return PixelsToScroll;
}

void QaConfig::setPixelOpts( int delay, int scroll )
{
    if ( delay <= 0 )
        delay = 1;
    ScrollDelay = delay;
    if ( scroll < 1 )
        scroll = 1;
    PixelsToScroll = scroll;
}


// Parse size

bool QaConfigParserSize::parseString( const QString &s, QaConfig *conf )
{
    QRegExp r( "^Size\\s=\\s.*" );
    QRegExp r2( "[0-9]+" );

    if ( r.match( s ) != -1 )
    {
        int offs, len;
        offs = r2.match( s, 0, &len );
        if ( offs != -1 )
        {
            QString match = s.mid( offs, len );
            conf->setWindowWidth( match.toInt() );
            offs = r2.match( s, offs + len, &len );
            if ( offs != -1 )
            {
                QString match = s.mid( offs, len );
                conf->setWindowHeight( match.toInt() );
                return true;
            }
        }
        debug( "QaConfigParserSize::parseString() Error reading in size\n" );
    }
    return false;
}

bool QaConfigParserSize::save( QTextStream &s, QaConfig *conf )
{
    s << "Size = " << conf->windowSize().width() << ", " << conf->windowSize().height() << "\n";
    return true;
}

// Parse log size

bool QaConfigParserLogSize::parseString( const QString &s, QaConfig *conf )
{
    QRegExp r( "^LogSize\\s=\\s.*" );
    QRegExp r2( "[0-9]+" );

    if ( r.match( s ) != -1 )
    {
        int offs, len;
        QSize log( -1, -1 );
        offs = r2.match( s, 0, &len );
        if ( offs != -1 )
        {
            QString match = s.mid( offs, len );
            log.setWidth( match.toInt() );
            offs = r2.match( s, offs + len, &len );
            if ( offs != -1 )
            {
                QString match = s.mid( offs, len );
                log.setHeight( match.toInt() );
                conf->setLogSize( log );
                return true;
            }
        }
        qDebug( "QaConfigParserLogSize::parseString() Error reading in log size\n" );
    }
    return false;
}

bool QaConfigParserLogSize::save( QTextStream &s, QaConfig *conf )
{
    if ( conf->logSize() != QSize( -1, -1 ) )
        s << "LogSize = " << conf->logSize().width() << ", " << conf->logSize().height() << "\n";
    return true;
}

// Parse scroll options

bool QaConfigParserScroll::parseString( const QString &s, QaConfig *conf )
{
    QRegExp r( "^Scroll\\s=\\s.*" );
    QRegExp r2( "[0-9]+" );

    if ( r.match( s ) != -1 )
    {
        int offs, len;
        QSize log( 10, 3 );
        offs = r2.match( s, 0, &len );
        if ( offs != -1 )
        {
            QString match = s.mid( offs, len );
            log.setWidth( match.toInt() );
            offs = r2.match( s, offs + len, &len );
            if ( offs != -1 )
            {
                QString match = s.mid( offs, len );
                log.setHeight( match.toInt() );
                conf->setPixelOpts( log.width(), log.height() );
                return true;
            }
        }
        qDebug( "QaConfigParserScroll::parseString() Error reading in scroll options" );
    }
    return false;
}


bool QaConfigParserScroll::save( QTextStream &s, QaConfig *conf )
{
    s << "Scroll = " << conf->scrollDelay() << ", " << conf->scrollPixels() << "\n";
    return true;
}

// Parse smooth GUI

bool QaConfigParserSmooth::parseString( const QString &s, QaConfig *conf )
{
    QRegExp r( "^SmoothGUI\\s=\\s.*" );
    QRegExp r2( "[yY][eE][sS]" );
    QRegExp r3( "[nN][oO]" );

    if ( r.match( s ) != -1 )
    {
        int offs, len;
        offs = r2.match( s, 0, &len );
        if ( offs != -1 )
        {
            conf->setSmoothGUI( true );
            return true;
        }
        else
        {
            offs = r3.match( s, 0, &len );
            if ( offs != -1 )
            {
                conf->setSmoothGUI( false );
                return true;
            }
        }
    }
    return false;
}

bool QaConfigParserSmooth::save( QTextStream &s, QaConfig *conf )
{
    s << "SmoothGUI = ";
    s << conf->boolToAscii( conf->smoothGUI() );
    s << "\n";
    return true;
}

// Parse splitter

bool QaConfigParserSplitter::parseString( const QString &s, QaConfig *conf )
{
    QRegExp r( "^Splitter\\s=\\s*[0-9]+\\s*,\\s*[0-9]+" );
    QRegExp r2( "[0-9]+" );

    if ( r.match( s ) != -1 )
    {
        int offs, len;
        offs = 0;
        QValueList<int> lst;
        while ( ( offs = r2.match( s, offs, &len ) ) != -1 )
        {
            QString match = s.mid( offs, len );
            lst.append( match.toInt() );
            offs += len;
        }
        conf->setSplitterSizes( lst );
    }
    return false;
}

bool QaConfigParserSplitter::save( QTextStream &s, QaConfig *conf )
{
    s << "Splitter = ";
    int count = 0;
    QValueList<int> split = conf->splitterSizes();
    for ( QValueList<int>::Iterator i = split.begin(); i != split.end(); ++i, ++count )
    {
        if ( count > 0 )
            s << ", ";
        s << *i;
    }
    s << "\n";
    return true;
}


// Parse divider

bool QaConfigParserDivider::parseString( const QString &s, QaConfig *conf )
{
    QRegExp r( "^Divider\\s=\\s*[0-9]+\\s*,\\s*[0-9]+" );
    QRegExp r2( "[0-9]+" );

    if ( r.match( s ) != -1 )
    {
        int offs, len;
        offs = 0;
        QValueList<int> lst;
        while ( ( offs = r2.match( s, offs, &len ) ) != -1 )
        {
            QString match = s.mid( offs, len );
            lst.append( match.toInt() );
            offs += len;
        }
        conf->setDividerSizes( lst );
    }
    return false;
}

bool QaConfigParserDivider::save( QTextStream &s, QaConfig *conf )
{
    s << "Divider = ";
    int count = 0;
    QValueList<int> split = conf->dividerSizes();
    for ( QValueList<int>::Iterator i = split.begin(); i != split.end(); ++i, ++count )
    {
        if ( count > 0 )
            s << ", ";
        s << *i;
    }
    s << "\n";
    return true;
}

// Parse dir/hot - list

bool QaConfigParserDirHot::parseString( const QString &s, QaConfig *conf )
{
    QRegExp r( "^HotListFirst\\s=\\s.*" );
    QRegExp r2( "[yY][eE][sS]" );

    if ( r.match( s ) != -1 )
    {
        if ( r2.match( s ) != -1 )
            conf->setHotDir( true );
        else
            conf->setHotDir( false );
    }
    return false;
}

bool QaConfigParserDirHot::save( QTextStream &s, QaConfig *conf )
{
    QString HotDir;
    if ( conf->hotDir() )
        HotDir = "Yes";
    else
        HotDir = "No";
    s << "HotListFirst = " << conf->boolToAscii( conf->hotDir() ) << "\n";
    return true;
}

// Parse FullScreen

bool QaConfigParserFull::parseString( const QString &s, QaConfig *conf )
{
    QRegExp r( "^FullScreen\\s=\\s.*" );
    QRegExp r2( "[yY][eE][sS]" );

    if ( r.match( s ) != -1 )
    {
        if ( r2.match( s ) != -1 )
            conf->setFullScreen( true );
        else
            conf->setFullScreen( false );
    }
    return false;
}

bool QaConfigParserFull::save( QTextStream &s, QaConfig *conf )
{
//      QString HotDir;
//      if ( conf->fullScreen() )
//          HotDir = "Yes";
//      else
//          HotDir = "No";
    s << "FullScreen = " << conf->boolToAscii( conf->fullScreen() ) << "\n";
    return true;
}

// Parse fastDir

bool QaConfigParserFast::parseString( const QString &s, QaConfig *conf )
{
    QRegExp r( "^FastDir\\s=\\s.*" );
    QRegExp r2( "[yY][eE][sS]" );

    if ( r.match( s ) != -1 )
    {
        if ( r2.match( s ) != -1 )
            conf->setFastDir( true );
        else
            conf->setFastDir( false );
    }
    return false;
}

bool QaConfigParserFast::save( QTextStream &s, QaConfig *conf )
{
    QString HotDir;
    if ( conf->fastDir() )
        HotDir = "Yes";
    else
        HotDir = "No";
    s << "FastDir = " << conf->boolToAscii( conf->fastDir() ) << "\n";
    return true;
}
