//
// $Id: qaconfig.hpp,v 1.3 1999/12/03 00:22:05 amos Exp $
//
// qaconfig.hpp
//
// Jan Borsodi <amos@abn.hibu.no>
// Created on: <25-Apr-1999 23:16:48 amos>
// Last change: Time-stamp: <03-Dec-1999 01:48:18 amos>
//
//

#ifndef QACONFIG_HPP
#define QACONFIG_HPP

#include <qstring.h>
#include <qsize.h>
#include <qlist.h>
#include <qstringlist.h>
#include <qvaluelist.h>
#include <qtextstream.h>

class QaConfigParser;

class QaConfig
{
public:
    virtual ~QaConfig();
    static QaConfig *instance();

    void setWindowSize( const QSize &s );
    void setWindowWidth( const int s );
    void setWindowHeight( const int s );
    void setLogSize( const QSize &s );
    void setHotList( const QStringList &l );
    void addToHotList( const QString &s );
    void setNameHotList( const QStringList &l );
    void addToNameHotList( const QString &s );
    void setHotDir( bool b );
    void setSmoothGUI( bool b );
    void setFastDir( bool b );
    void setFullScreen( bool b );
    void setSplitterSizes( const QValueList<int> &v );
    void setDividerSizes( const QValueList<int> &v );
    void setPixelOpts( int delay, int scroll );

    const QValueList<int> &splitterSizes() const;
    const QValueList<int> &dividerSizes() const;
    const QSize &windowSize() const;
    const QSize &logSize() const;
    const QString &name() const;
    const QString &hotName() const;
    const QString &absHotName() const;
    const QStringList &hotList() const;
    const QStringList &nameHotList() const;
    bool hotDir() const;
    bool smoothGUI() const;
    bool fastDir() const;
    bool fullScreen() const;
    int scrollDelay() const;
    int scrollPixels() const;

    QString boolToAscii( bool b ) const;

    void parseHotList();
    void save();
    /*!
      Returns the class name.

      Good for debugging.
    */
    virtual const char * isA() const { return "QaConfig"; };

protected:
	QaConfig();

private:
    QSize		WindowSize;
    QSize		LogSize;
    QString		Name;
    QString		AbsName;
    QString		HotName;
    QString		AbsHotName;
    QStringList		HotList;
    QStringList		NameHotList;
    bool		bHotDir;
    bool		bSmoothGUI;
    bool		bFastDir;
    bool		bFullScreen;
    QList<QaConfigParser> ParseList;
    QValueList<int>	SplitterSizes;
    QValueList<int>	DividerSizes;
    int			ScrollDelay;
    int			PixelsToScroll;

    static QaConfig *_instance;
};

class QaConfigParser
{
public:
    virtual bool parseString( const QString &s, QaConfig *conf ) = 0;
    virtual bool save( QTextStream &s, QaConfig *conf ) = 0;
};

class QaConfigParserScroll : public QaConfigParser
{
public:
    virtual bool parseString( const QString &s, QaConfig *conf );
    virtual bool save( QTextStream &s, QaConfig *conf );
};

class QaConfigParserSize : public QaConfigParser
{
public:
    virtual bool parseString( const QString &s, QaConfig *conf );
    virtual bool save( QTextStream &s, QaConfig *conf );
};

class QaConfigParserLogSize : public QaConfigParser
{
public:
    virtual bool parseString( const QString &s, QaConfig *conf );
    virtual bool save( QTextStream &s, QaConfig *conf );
};

class QaConfigParserSmooth : public QaConfigParser
{
public:
    virtual bool parseString( const QString &s, QaConfig *conf );
    virtual bool save( QTextStream &s, QaConfig *conf );
};

class QaConfigParserSplitter : public QaConfigParser
{
public:
    virtual bool parseString( const QString &s, QaConfig *conf );
    virtual bool save( QTextStream &s, QaConfig *conf );
};

class QaConfigParserDivider : public QaConfigParser
{
public:
    virtual bool parseString( const QString &s, QaConfig *conf );
    virtual bool save( QTextStream &s, QaConfig *conf );
};

class QaConfigParserDirHot : public QaConfigParser
{
public:
    virtual bool parseString( const QString &s, QaConfig *conf );
    virtual bool save( QTextStream &s, QaConfig *conf );
};

class QaConfigParserFull : public QaConfigParser
{
public:
    virtual bool parseString( const QString &s, QaConfig *conf );
    virtual bool save( QTextStream &s, QaConfig *conf );
};

class QaConfigParserFast : public QaConfigParser
{
public:
    virtual bool parseString( const QString &s, QaConfig *conf );
    virtual bool save( QTextStream &s, QaConfig *conf );
};

#endif // QACONFIG_HPP
