// QaThumbnailListview.hpp
//
// Jan Borsodi <jbfdata@online.no>
// Last change: Time-stamp: <30-Jul-1999 04:40:41 amos>
//
//

#ifndef QATHUMBNAILLISTVIEW_HPP
#define QATHUMBNAILLISTVIEW_HPP

#include <qscrollview.h>
#include <qlist.h>
#include <qpixmap.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qglist.h>
#include <qstring.h>
#include <qpainter.h>
#include <qevent.h>
#include <qpoint.h>
#include <qrect.h>
#include <qsize.h>
#include <qwidget.h>

class QaThumbnailListview;

class QaThumbnailItem
{
public:
    QaThumbnailItem( const QaThumbnailListview *parent );
    virtual ~QaThumbnailItem();

    friend QaThumbnailListview;
    /*! \fn virtual const char * isA() const
      Returns the class name.

      Good for debugging.
    */
    virtual const char * isA() const { return "QaThumbnailItem"; };

    virtual const char *text() { return Text; }
    virtual QSize size() const = 0;
    virtual int width() const = 0;
    virtual int height() const = 0;
    bool isSelected() const { return Selected; }

protected:
    virtual void paint( QPainter * ) = 0;
    void setText( const char *t );
    void setSelected( bool s );
    const QaThumbnailListview *parent() const { return Parent; }

private:
    QString Text;
    bool    Selected;
    const QaThumbnailListview *Parent;
};


class QaThumbnailPreview : QaThumbnailItem
{
public:
	QaThumbnailPreview( const QaThumbnailListview *parent, QFileInfo *info );
	QaThumbnailPreview( const QaThumbnailListview *parent, QFileInfo *info, const QPixmap &p );
	virtual ~QaThumbnailPreview();

	friend QaThumbnailListview;
	/*! \fn virtual const char * isA() const
	  Returns the class name.

	  Good for debugging.
	*/
	virtual const char * isA() const { return "QaThumbnailPreview"; };

	virtual QSize size() const { return Size; }
	virtual int width() const { return Size.width(); }
	virtual int height() const { return Size.height(); }
	void setSize( const QSize &s );
	void setLineWidth( const int s );
	int lineWidth() const { return LWidth; }
	void setTextSpace( const int s );
	int textSpace() const { return TSpace; }
	void setTextFramespace( const int s );
	int textFramespace() const { return TFSpace; }
	QRect pixmapRect() const;
	QRect pixmapFrameRect() const;
	QRect filenameRect() const;
	QRect filenameFrameRect() const;
	const QPixmap &pixmap() const { return Pix; }
	void setPixmap( const QPixmap &p );
	QPoint pixmapPos() const;
	const QFileInfo *fileInfo() const { return FInfo; }

protected:
	virtual void paint( QPainter * );

private:
	int LWidth;
	int TSpace, TFSpace;
	QSize Size;
	QPixmap Pix;
	QFileInfo *FInfo;
};


class QaThumbnailListview : public QScrollView
{
	Q_OBJECT
public:
	QaThumbnailListview( QWidget *parent = 0, const char *name = 0 );
	virtual ~QaThumbnailListview();

	/*! \fn virtual const char * isA() const
	  Returns the class name.

	  Good for debugging.
	*/
	virtual const char * isA() const { return "QaThumbnailListview"; };

	QSize viewportSize();
	QSize itemSpaceSize() const { return ItemSpace; }
	void setItemSpaceSize( const QSize &s );
	QSize maxItemSize() const { return ItemSize; }
	QSize itemMatrix() const { return ItemMatrix; }
	void setDirectory( const QFileInfoList &f );
	void addToDirList( const QFileInfo *fi );
	void addToImageList( const QFileInfo *fi );
	void clear();
	void scanDirectory();
	void recalcMatrix();
	bool directoryCached() const { return DirCached; }
	QaThumbnailItem *itemAt( int x, int y );

	void insertItem( QaThumbnailItem *item, int index = -1 );

signals:
	void doubleClicked( QaThumbnailItem * );

protected:
	virtual void drawContentsOffset( QPainter *p, int ox, int oy, int cx, int cy, int cw, int ch );
	virtual void resizeEvent( QResizeEvent *e );
	virtual void viewportMousePressEvent( QMouseEvent *e );
	virtual void viewportMouseDoubleClickEvent( QMouseEvent *e );
	bool directoryDiffers( const QDir &d );
	
private:
	QList<QaThumbnailItem> *SelectList;
	QSize ItemSpace;
	QSize ItemSize;
	QSize ItemMatrix;
	QList<QaThumbnailItem> ItemList;
	QFileInfoList DirList;
	QFileInfoList ImageList;
	bool DirCached;
	bool dirty;
};

#endif // QATHUMBNAILLISTVIEW_HPP
