/* qmimageview.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <30-Jul-1999 04:44:54 amos>
 */

#include "qmimageview.h"
#include "qmmainwindow.h"
#include "qaconfig.hpp"

#include <iostream.h>
#include <math.h>
#include <stdlib.h>

#include <qapplication.h>
#include <qcursor.h>
#include <qdatetime.h>
#include <qdragobject.h>
#include <qfileinfo.h>
#include <qkeycode.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qtimer.h>

#include <qpixmap.h>
#include <qstring.h>
#include <qevent.h>
#include <qfile.h>
#include <qimage.h>
#include <qpoint.h>
#include <qrect.h>
#include <qscrollview.h>
#include <qsize.h>
#include <qwidget.h>
#include <qwmatrix.h>
#include <iostream>
#include <qcolor.h>
#include <qbrush.h>
#include <qframe.h>

/* XPM */
const char * picture_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #000000",
"X	c #0000FF",
"XXXXXXXX        ",
"XXXXXXXX        ",
"XXXXXXXX        ",
"XXXXXXXX   X    ",
"XXXXXXXX  XXX   ",
"         XXXXX  ",
"        XXXXXXX ",
"       XXXXXXXXX",
"   XXX          ",
" XXXXXXX        ",
"XXXXXXXXX       ",
"XXXXXXXXX       ",
"XXXXXXXXX       ",
" XXXXXXX        ",
"   XXX          ",
"                "};

extern QLabel *g_pLargeInfo;
extern QLabel *g_pSmallInfo;

//  static int ScrollDelay    = 10;
//  static int PixelsToScroll = 3;

/*
 TODO:

 Hide cursor if it has been still for more than, say, 3 seconds.
  
 */

/*!
  \class QmImageView qmimageview.h
  \brief The QmImageView class shows a picture in a scrollable area.

  This class is meant for showing images in their original size.

  \bug The keyboard scrolling is buggy.
  \bug The drag and drop of "itself" must be fixed (i.e. not accept drops from
       itself.)
*/

/*!

*/

QmImageView::QmImageView(
    QWidget *Parent,
    const char *Name)
    : QScrollView(Parent, Name)
{
    ButtonReleased = false;

    setVScrollBarMode(AlwaysOff);
    setHScrollBarMode(AlwaysOff);

    BlackBackground.setStyle(SolidPattern);
    BlackBackground.setColor(black);
    setFrameStyle( QFrame::NoFrame );
    viewport()->setBackgroundMode( NoBackground );
    setMargins( 0, 0, 0, 0 );

    setFocusPolicy(QWidget::WheelFocus);

    for(int i = 0; i < 4; i++)
    {
        DirectionPressed[i] = false;
    }
    HScrollDir = HNone;
    VScrollDir = VNone;
    
    ShiftPressed = false;

    pTimer = new QTimer(this);
    connect(pTimer, SIGNAL(timeout()), this, SLOT(timerDone()));

    setAcceptDrops( true );

    // Check QWidget::setBackgroundMode(NoBackground) if flickering
    // becomes a problem.

//    setBackgroundMode(NoBackground);
//    setBackgroundColor(black);
}


/*!

*/

QmImageView::~QmImageView()
{
}


/*!
  Loads the image given with \a Filename. Returns true if successful,
  false otherwise.  
*/

bool
QmImageView::loadImage(
    const char *Filename)
{
    CurrentZoom = 100;
    ImageFilename = Filename;

    QTime t;
    t.start();

    if( ! ThePixmap.load(Filename))
	return false;

    PixmapToShow = ThePixmap;

    setContentsPos(0, 0);
    resizeContents(PixmapToShow.width(), PixmapToShow.height());

    // Repaint the _drawing_ area

    viewport()->repaint();

    CHECK_PTR(g_pLargeInfo);
    CHECK_PTR(g_pSmallInfo);

    QString s;
    s.sprintf("%dx%d", ThePixmap.width(), ThePixmap.height());
    g_pSmallInfo->setText((const char *)s);

    QFileInfo fi(Filename);
    s.sprintf("%s (%s format) loaded in %0.1f second(s)",
	      (const char *)fi.fileName(),
//	      (ThePixmap.depth() == 32 ? 24 : ThePixmap.depth()),
	      QImage::imageFormat(Filename),
	      (float)t.elapsed() / 1000.0);
    g_pLargeInfo->setText((const char *)s);

    return true;
}


void
QmImageView::setPixmap(
    const QPixmap &Pixmap)
{
    CurrentZoom = 100;
    ThePixmap = Pixmap;
    PixmapToShow = ThePixmap;
    setContentsPos(0, 0);
    resizeContents(PixmapToShow.width(), PixmapToShow.height());

    // Repaint the _drawing_ area
    
    viewport()->repaint();
}


int
QmImageView::currentZoom(void) const
{
    return CurrentZoom;
}


/*!
  Zooms the image by \a Percent percent. If \a Percent is negative, the
  image is scaled down by that percent.
*/

void
QmImageView::zoom(
    int Percent)
{
    ASSERT(Percent <= 300);
    ASSERT(Percent >= 10);

    CurrentZoom = Percent;
    
    double Zoom = Percent / 100.0;

    QWMatrix m;
    m.scale(Zoom, Zoom);
    PixmapToShow = ThePixmap.xForm(m);
   
    resizeContents(PixmapToShow.width(), PixmapToShow.height());

    // Repaint the _drawing_ area
    viewport()->repaint();
}


/*!

 */

void
QmImageView::keyPressEvent(
    QKeyEvent *e)
{
    switch(e->key())
    {
        case Key_Down:
            VScrollDir = VDown;
            DirectionPressed[0] = true;
	
            if( ! pTimer->isActive())
                pTimer->start(QaConfig::instance()->scrollDelay(), true);
            break;
	
        case Key_Up:
            VScrollDir = VUp;
            DirectionPressed[1] = true;
	
            if( ! pTimer->isActive())
                pTimer->start(QaConfig::instance()->scrollDelay(), true);
            break;
	
        case Key_Left:
            HScrollDir = HLeft;
            DirectionPressed[2] = true;
	
            if( ! pTimer->isActive())
                pTimer->start(QaConfig::instance()->scrollDelay(), true);
            break;

        case Key_Right:
            HScrollDir = HRight;
            DirectionPressed[3] = true;
	
            if( ! pTimer->isActive())
                pTimer->start(QaConfig::instance()->scrollDelay(), true);
            break;

        case Key_Shift:
            ShiftPressed = true;
            break;
	    
//  		case Key_Delete:
//  			deleteImage();
//  			break;

        case Key_Space:
//              emit queryNextImageLoop();
//              break;
        case Key_PageDown:
            emit queryNextImage();
            break;

        case Key_Backspace:
//              emit queryPrevImageLoop();
//              break;
        case Key_PageUp:
            emit queryPrevImage();
            break;

        case Key_Plus:
            if ( CurrentZoom + 20 <= 300 )
                zoom( CurrentZoom + 20 );
            break;
        case Key_Minus:
            if ( CurrentZoom - 20 >= 10 )
                zoom( CurrentZoom - 20 );
            break;

        default:
            e->ignore();
            return;
    }
    e->accept();
}


/*!

*/

void
QmImageView::keyReleaseEvent(
    QKeyEvent *e)
{
    switch(e->key())
    {
        case Key_Down:
            DirectionPressed[0] = false;
	
            if( ! directionPressed())
            {
                if(pTimer->isActive())
                    pTimer->stop();
	    
                VScrollDir = VNone;
            }
            else if(DirectionPressed[1] == true)
                VScrollDir = VUp;
            else
                VScrollDir = VNone;

            break;
	
        case Key_Up:
            DirectionPressed[1] = false;
	
            if( ! directionPressed())
            {
                if(pTimer->isActive())
                    pTimer->stop();
	    
                VScrollDir = VNone;
            }
            else if(DirectionPressed[0] == true)
                VScrollDir = VDown;
            else
                VScrollDir = VNone;
	
            break;
	
        case Key_Left:
            DirectionPressed[2] = false;
	
            if( ! directionPressed())
            {
                if(pTimer->isActive())
                    pTimer->stop();
	    
                HScrollDir = HNone;
            }
            else if(DirectionPressed[3] == true)
                HScrollDir = HRight;
            else
                HScrollDir = HNone;
	
            break;

        case Key_Right:
            DirectionPressed[3] = false;
	
            if( ! directionPressed())
            {
                if(pTimer->isActive())
                    pTimer->stop();
	    
                HScrollDir = HNone;
            }
            else if(DirectionPressed[2] == true)
                HScrollDir = HLeft;
            else
                HScrollDir = HNone;
	
            break;

        case Key_Shift:
            ShiftPressed = false;
	
            break;

        case Key_Escape:
        case Key_Enter:
        case Key_Return:
//            releaseKeyboard();
            emit returnPressed();
            break;

        default:
            e->ignore();
    }
}


/*!
  Returns true if a scroll key is pressed, false if not.
*/

bool
QmImageView::directionPressed(void) const
{
    for(int i = 0; i < 4; i++)
    {
	if(DirectionPressed[i] == true)
	    return true;
    }
    
    return false;
}

/*!

*/

void
QmImageView::leaveEvent(
    QEvent * /*e*/)
{
//     if( ! ButtonReleased)
//     {
//  	QImageDrag *d = new QImageDrag(TheImage, viewport());
//  	d->setPixmap(QPixmap(picture_xpm), QPoint(8, 8));
//  	d->dragCopy();
//     }
}


/*!

*/

void
QmImageView::viewportMousePressEvent(
    QMouseEvent *e)
{
//      grabKeyboard();
    if(e->button() == LeftButton)
    {
	// WARNING: QImageDrag does not take pixmaps as argument.
	// Converting from pixmap to image is slow. Write a QmPixmapDrag?

	
//    	QImageDrag *d = new QImageDrag(ThePixmap.convertToImage(), viewport());
//    	d->setPixmap(QPixmap(picture_xpm), QPoint(8, 8));
//    	d->dragCopy();
	

    }
//    parentWidget()->raise();

    if( contentsWidth() > width() && contentsHeight() > height() )
        setCursor(sizeAllCursor);
    else if(contentsWidth() <= width() && contentsHeight() > height())
        setCursor(sizeVerCursor);
    else if(contentsWidth() > width() && contentsHeight() <= height())
        setCursor(sizeHorCursor);
}


/*!

*/

void
QmImageView::viewportMouseReleaseEvent(
    QMouseEvent *)
{
    setCursor(arrowCursor);

    ButtonReleased = true;
}


/*!

*/

void
QmImageView::viewportMouseMoveEvent(
    QMouseEvent *e)
{
    static int preX = e->x();
    static int preY = e->y();

    int dx, dy;

    // We need to check if this is the first move after a new mouse click
    // has been done. This is so that delta x and delta y is to be correct.
    // If we don't check this, delta x and delta y will jump to great
    // numbers whenever the mouse is moved and pressed elsewhere.
    
    if(ButtonReleased)
    {
	dx = 0;
	dy = 0;

	ButtonReleased = false;
    }
    else
    {
	dx = e->x() - preX;
	dy = e->y() - preY;
    }

    // Multiply by 2 to speed up the scrolling
    
    scrollBy( -dx * 2, -dy * 2);

    preX = e->x();
    preY = e->y();
}


/*!
  Draws the image.
*/

void
QmImageView::drawContentsOffset(
    QPainter *p,
    int OffsetX, int OffsetY,
    int ClipX, int ClipY,
    int ClipW, int ClipH)
{ 
    p->setClipping( false );

    QRect clip( ClipX, ClipY, ClipW, ClipH );
    QRect image( PixmapToShow.rect() );


    // Four rects each representing the area outside the picture
    // Only in use when the picture is centered horizontally/vertically.
    QRect top, bottom;
    QRect left, right;
    // If image width or height is less than viewing size, make them centred.

    int xdelta = ( width() - image.width() )/2;
    int xdelta2 = ( width() + image.width() )/2 ;

    int ydelta = ( height() - image.height() )/2;
    int ydelta2 = ( height() + image.height() )/2 ;

    if ( image.width() < width() )
    {
        left = QRect( 0, 0, xdelta, height() );
        right = QRect( xdelta2, 0, xdelta, height() );
        image.moveBy( xdelta, 0 );
    }
    else
        image.moveBy( -OffsetX, 0 );

    if ( image.height() < height() )
    {
        if ( left.isNull() )
        {
            top = QRect( 0, 0, width(), ydelta );
            bottom = QRect( 0, ydelta2, width(), ydelta );
        }
        else
        {
            top = QRect( xdelta, 0, image.width(), ydelta );
            bottom = QRect( xdelta, ydelta2, image.width(), ydelta );
        }
        image.moveBy( 0, ydelta );
    }
    else
        image.moveBy( 0, -OffsetY );

    // Readjust for new origo
    clip.moveBy( -OffsetX, -OffsetY );

    // Draw the pixmap. Center it horizontally and vertically if the
    // image is smaller than the widget.

    QRect part = clip.intersect( image );
    QRect offs = part;
    part.moveBy( -image.x(), -image.y() );

    QBrush b( QColor( 120, 120, 120 ) );

    if ( !top.isNull() || !bottom.isNull() )
    {
        QRect r = top.intersect( clip );
        if ( !r.isNull() )
            p->fillRect( r, b );

        r = bottom.intersect( clip );
        if ( !r.isNull() )
            p->fillRect( r, b );
    }

    if ( !left.isNull() || !right.isNull() )
    {
        QRect r = left.intersect( clip );
        if ( !r.isNull() )
            p->fillRect( r, b );

        r = right.intersect( clip );
        if ( !r.isNull() )
            p->fillRect( r, b );
    }

    if ( !part.isNull() )
    {
        // Draw background if pixmap has a mask
        if ( PixmapToShow.mask() != 0 )
            p->fillRect( QRect( offs.topLeft(), part.size() ), b );

        p->drawPixmap( offs.topLeft(), PixmapToShow, part );
    }

}


/*!

 */

void
QmImageView::dragEnterEvent(
    QDragEnterEvent *)
{
//      qDebug( "QmImageView::dragEnterEvent()" );
    
//      // Check if you want the drag...
    
//      if(QTextDrag::canDecode( e ) || QImageDrag::canDecode( e ) )
//      {
//  		e->accept();
//      }


//      // Give the user some feedback...
    
//      QString t;
//      for( int i=0; e->format( i ); i++ ) {
//  		if ( *(e->format( i )) ) {
//  			if ( !t.isEmpty() )
//  				t += "\n";
//  			t += e->format( i );
//  		}
//      }
    
//  	emit message( t );
//  //     setBackgroundColor(white);
}


/*!

 */

void
QmImageView::dragMoveEvent(
    QDragMoveEvent *)
{
//  //      cout << "QmImageView::dragMoveEvent()\n";
//  	if ( QTextDrag::canDecode( e ) )
//  	{
//  		e->accept();
//  	}
//  	else if ( QImageDrag::canDecode( e ) )
//  	{
//  		e->accept();
//  	}
//  	else
//  	{
//  		e->ignore();
//  	}
//  //	e->ignore();
}


/*!

 */

void
QmImageView::dragLeaveEvent(
    QDragLeaveEvent *)
{
//  //	e->ignore();
//    qDebug( "QmImageView::dragLeaveEvent()" );

//      // Give the user some feedback...
//  //     emit message("");
//  //     setBackgroundColor(lightGray);
}


/*!

 */

void
QmImageView::dropEvent(
    QDropEvent *)
{
//      qDebug( "QmImageView::dropEvent()" );

//      // Try to decode to the data you understand...

//      QString str;
//      if(QTextDrag::decode(e, str))
//      {
//  			// Does the file exist (is it a file)?
	
//  		if(QFile::exists(str))
//  		{
//  			QImage img;

//  			if(img.load(str))
//  			{
//  				ThePixmap = img;

//  				resizeContents(PixmapToShow.width(), PixmapToShow.height());

//  					// Repaint the _drawing_ area
    
//  				viewport()->repaint();
//  			}
//  		}
//  		e->accept();
//  		return;
//      }

//      QImage img;
//      if(QImageDrag::decode(e, img))
//      {
//  		ThePixmap = img;
//  		resizeContents(PixmapToShow.width(), PixmapToShow.height());
//  		viewport()->repaint();
//  		e->accept();
//  		return;
//      }
//  	e->ignore();
}


/*!
  This function ensures smooth scrolling, it is called every 10 or so millisecond
  when any of the scroll keys are pressed down. It will scroll the view by
  approximately 1 to 9 pixels per tick.

  TODO:
  It might be necessary to change the update time depending on refresh rate,
  machine power etc... Maybe a slider in the configuration.
*/

void
QmImageView::timerDone()
{
    int DeltaX, DeltaY, PixVal;

    if(ShiftPressed)
        PixVal = QaConfig::instance()->scrollPixels();
    else
        PixVal = QaConfig::instance()->scrollPixels() * 3;

    switch(VScrollDir)
    {
        case VDown:
            DeltaY = PixVal;
            break;
	
        case VUp:
            DeltaY = -PixVal;
            break;
	
        case VNone:
        default:
            DeltaY = 0;
            break;
    }
		
    switch(HScrollDir)
    {
        case HLeft:
            DeltaX = -PixVal;
            break;
	
        case HRight:
            DeltaX = PixVal;
            break;
	
        case HNone:
        default:
            DeltaX = 0;
            break;
    }
    
    scrollBy(DeltaX, DeltaY);

    pTimer->start(QaConfig::instance()->scrollDelay(), true);
//    pTimer->stop();
}

void
QmImageView::deleteImage(void)
{
    QmMainWindow::deleteItem((const char *)ImageFilename);
}

void QmImageView::focusInEvent( QFocusEvent * )
{
    qDebug( "QmImageView::focusInEvent(QFocusEvent *)" );
}

void QmImageView::focusOutEvent( QFocusEvent * )
{
    qDebug( "QmImageView::focusOutEvent(QFocusEvent *)" );
}

void QmImageView::wheelEvent( QWheelEvent *e )
{
    if ( e->delta() > 0 )
        emit queryNextImage();
    else if ( e->delta() < 0 )
        emit queryPrevImage();
}
