/* qmmainwindow.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <03-Dec-1999 02:09:35 amos>
 */

#include <iostream.h>

#include "qmbrowser.h"
#include "qmimageview.h"
#include "qmimageviewbrowser.h"
#include "qmlistdialog.h"
#include "qmmainwindow.h"
#include "strings.h"
#include "version.h"
#include "qmlistviewitem.h"
#include "qacommonlist.hpp"
#include "qaconfig.hpp"

#include <qaccel.h>
#include <qapplication.h>
#include <qcursor.h>
#include <qcombobox.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qprogressbar.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <qvalidator.h>
#include <qwhatsthis.h>
#include <qwidgetlist.h>
#include <fstream.h>
#include <qlabel.h>
#include <qframe.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qdir.h>
#include <qfile.h>
#include <qlist.h>
#include <qmainwindow.h>
#include <qpoint.h>
#include <qpopupmenu.h>
#include <qsize.h>
#include <qtimer.h>
#include <qtoolbar.h>
#include <qwidget.h>
#include <qwidgetstack.h>
#include <qmenubar.h>
#include <qlistview.h>
#include <fstream>
#include <iostream>
#include <qsplitter.h>

#include <qmotifstyle.h>
#include <qcdestyle.h>
#include <qwindowsstyle.h>
#include <qplatinumstyle.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qvgroupbox.h>
#include <qpushbutton.h>
#include <qvbox.h>
#include <qslider.h>
#include <qbrush.h>
#include <qcolor.h>
#include <qregion.h>
#include <qrect.h>
#include <qevent.h>


QList<QmMainWindow>	QmMainWindow::WindowList;
QPopupMenu		*QmMainWindow::pWindowMenu	= 0;

#ifdef DEBUG
extern QTextView *g_pLog;
#endif

/* Global variables:
 *
 * g_pProgress		is the progress bar in the statusbar.
 * g_pSmallInfo		is a tiny text field in the statusbar. Use it for
 *			filecounts etc.
 * g_pLargeInfo		is the large text field in the statusbar. Use it
 *			for longer descriptions.
 * g_pImageViewBrowser	is the fullsize image viewer.
 * g_pCommonList  is the common list for the the three listviews.
 */

QProgressBar		*g_pProgress			= 0;
QLabel			*g_pSmallInfo			= 0;
QLabel			*g_pLargeInfo			= 0;
QmImageViewBrowser	*g_pImageViewBrowser		= 0;
QaCommonList	*g_pCommonList = 0;

/*!
  \class QmMainWindow qmmainwindow.h
  \brief The QMainWindow is the main window.

  This window is the main working window.

  \warning
  
  It is \e vital that you do \e not create a new window on
  the stack. This is due to the multi-window feature. \e All objects of
  this class \e must be created with \c new. If you create a object of this
  class on the stack, the application \e will crash when the window is closed.
*/

/*!
  Creates the window.

  The \a Parent and \a Name arguments are sent to the QMainWindow constructor.

  \warning
  
  Read the detailed information!
*/

QmMainWindow::QmMainWindow(
    QWidget *Parent,
    const char *Name)
    : QMainWindow(Parent, Name)
{
//      QRegion region( QRect( 0, 0, width(), height() ),
//                      QRegion::Rectangle );
//      setMask( region );
    pOptions = 0;
    pOldFocus = 0;
    MenuBarHidden = false;
    ToolBarHidden = false;
    StatusBarHidden = false;

    FullMenuBarHidden = true;
    FullToolBarHidden = true;
    FullStatusBarHidden = true;

    IsFullScreen = QaConfig::instance()->fullScreen();

    // If this is the very first window created, at it to the list here.
    // Otherwise, it has already been added in the newWindow() function.
//    if(WindowList.count() == 0)
    {
        WindowList.setAutoDelete(false);
        WindowList.append(this);
    }

    // Create a log window
    // Note: Only for debug mode
#ifdef DEBUG
    g_pLog = pLog = new QTextView( 0, "log-view" );
    pLog->setCaption( "QtVu - Error Log" );
    pLog->setPaper( QBrush( QColor( "DarkSlateBlue" ) ) );

    QAccel *pLogAccels = new QAccel(pLog);
    pLogAccels->connectItem( pLogAccels->insertItem(CTRL + Key_L),
                             this, SLOT(toggleLog()));
    pLogAccels->connectItem(pLogAccels->insertItem(CTRL + Key_Q),
                            this, SLOT(quit()));

    if ( QaConfig::instance()->logSize() == QSize( -1, -1 ) )
        pLog->resize( 100, 200 );
    else
        pLog->resize( QaConfig::instance()->logSize() );
//    pLog->show();
#endif
    g_pCommonList = new QaCommonList();

    QString Caption;
    Caption.sprintf("%s - Window %d", APP_NAME, WindowList.count());
    setCaption((const char *)Caption);

    QAccel *pAccels = 0;

    pStack = new QWidgetStack(this, "main-widgetstack");

    pBrowser = new QmBrowser(pStack, "main-browser");
    {
        connect(pBrowser, SIGNAL(fileSelected(const char *)),
                this, SLOT(showImage(const char *)));

        connect(pBrowser, SIGNAL(imageSelected(const QPixmap &)),
                this, SLOT(showImage(const QPixmap &)));

        connect(pBrowser, SIGNAL(message(const QString &)),
                statusBar(), SLOT(message(const QString &)));
    }

    /*
    pImageView = new QmImageView(pStack);
    {
	pImageView->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);

	connect(pImageView, SIGNAL(message(const char *)),
		statusBar(), SLOT(message(const char *)));

	connect(pImageView, SIGNAL(returnPressed()),
		this, SLOT(toggleView()));

	connect(pImageView, SIGNAL(queryNextImage()),
		this, SLOT(nextImage()));

	connect(pImageView, SIGNAL(queryPrevImage()),
		this, SLOT(prevImage()));

	QWhatsThis::add(pImageView, WT_IMAGE_VIEW);
    }
    */

//      pImageView = 0;
    g_pImageViewBrowser = new QmImageViewBrowser(pStack,"main-imageviewbrowser");

    connect( g_pImageViewBrowser->imageView(), SIGNAL( queryNextImage() ),
             pBrowser, SLOT( queryNextImage() ) );

    connect( g_pImageViewBrowser->imageView(), SIGNAL( queryPrevImage() ),
             pBrowser, SLOT( queryPrevImage() ) );

    connect( g_pImageViewBrowser->imageView(), SIGNAL( returnPressed() ),
             this, SLOT( toggleView() ) );


    pStack->addWidget(pBrowser, BrowserID);
//    pStack->addWidget(pImageView, ImageViewID);
    pStack->addWidget(g_pImageViewBrowser, ImageViewID);
    pStack->raiseWidget(BrowserID);

    setCentralWidget(pStack);

    pToolBar = new QToolBar(this, "Main toolbar");
    {
	// Unfortunately, we can not simply set the toolbar's focus policy
	// to NoFocus and hope that the toolbuttons in it won't be able
	// to "tab to". Bug? We need to set NoFocus on all widgets in a toolbar
	// instead.
	
//	pToolBar->setFocusPolicy(QWidget::NoFocus);

        QString file = getIconPath( "fileopen.xpm" );
        QPixmap openIcon(file);
        pFileOpen = new QToolButton(
            openIcon,
            "Tooltip",
//              "Statusbar",
            0,
            this, SLOT(toggleView()),
            pToolBar);

        pFileOpen->setToggleButton(true);
        pFileOpen->setFocusPolicy(QWidget::NoFocus);

        file = getIconPath( "back.xpm" );
        QPixmap BackIcon( file );
        pBack = new QToolButton(
            BackIcon,
            "Previous image",
//              "Shows the previous image.",
            0,
            pBrowser, SLOT(queryPrevImage()),
            pToolBar);

        pBack->setFocusPolicy(QWidget::NoFocus);
    
        file = getIconPath( "forward.xpm" );
        QPixmap forwardIcon( file );
        pForward = new QToolButton(
            forwardIcon,
            "Next image (Ctrl+F)",
//              "Shows the next image.",
            0,
            pBrowser, SLOT(queryNextImage()),
            pToolBar);

        pForward->setFocusPolicy(QWidget::NoFocus);
    
        pSlideshowTimer = new QTimer(this);
        connect(pSlideshowTimer, SIGNAL(timeout()),
                pBrowser, SLOT(queryNextImage()));
	
        file = getIconPath( "slide.xpm" );
        QPixmap slideIcon( file );
        pSlide = new QToolButton(
            slideIcon,
            "Slideshow",
//              "Shows the images in a slideshow.",
            0,
            this, SLOT(toggleSlideshow()),
            pToolBar);

        pSlide->setToggleButton(true);
        pSlide->setFocusPolicy(QWidget::NoFocus);

        QString s;
        s.sprintf("Zooms up the picture by %d%%", ZoomStep);
        file = getIconPath( "zoomin.xpm" );
        QPixmap zoomInIcon( file);
        pZoomIn = new QToolButton(
            zoomInIcon,
            (const char *)s,
//              (const char *)s,
            0,
            this, SLOT(zoomUp()),
            pToolBar);

        pZoomIn->setFocusPolicy(QWidget::NoFocus);
	
        s.sprintf("Zooms down the picture by %d%%", ZoomStep);
        file = getIconPath( "zoomout.xpm" );
        QPixmap zoomOutIcon( file );
        pZoomOut = new QToolButton(
            zoomOutIcon,
            (const char *)s,
//              (const char *)s,
            0,
            this, SLOT(zoomDown()),
            pToolBar);

        pZoomOut->setFocusPolicy(QWidget::NoFocus);

		/*
		  pZoom = new QComboBox(false, pToolBar);
		  {
		  pZoom->insertItem("20");
		  pZoom->insertItem("40");
		  pZoom->insertItem("60");
		  pZoom->insertItem("80");
		  pZoom->insertItem("100");
		  pZoom->insertItem("120");
		  pZoom->insertItem("140");
		  pZoom->insertItem("160");
		  pZoom->insertItem("180");
		  pZoom->insertItem("200");

		  pZoom->setFixedWidth(pZoom->sizeHint().width());
//	    pZoom->setValidator(new QIntValidator(20, 300, pZoom));

connect(pZoom, SIGNAL(activated(const char *)),
this, SLOT(zoomEntered(const char *)));
}
	*/

	pToolBar->addSeparator();
    
	QToolButton *pWhatsThis = QWhatsThis::whatsThisButton(pToolBar);
	pWhatsThis->setFocusPolicy(QWidget::NoFocus);
	
	QWhatsThis::add(pFileOpen,	WT_IMAGE_VIEW );
	QWhatsThis::add(pBack,		WT_BACK		);
	QWhatsThis::add(pForward,	WT_FORWARD	);
	QWhatsThis::add(pZoomIn,	WT_ZOOM_IN	);
	QWhatsThis::add(pZoomOut,	WT_ZOOM_OUT	);
	QWhatsThis::add(pSlide,		WT_SLIDE_SHOW	);
    }

    QPopupMenu *pFileMenu = new QPopupMenu;
    {
        menuBar()->insertItem("&File", pFileMenu);

        pFileMenu->insertItem("&New",
                              this, SLOT(newMainWindow()), CTRL + Key_N);
        pFileMenu->insertSeparator();
        pFileMenu->insertItem("&Close",
                              this, SLOT(closeWindow()), CTRL + Key_W);
        pFileMenu->insertSeparator();
        pFileMenu->insertItem("&Quit", this, SLOT(quit()), CTRL + Key_Q);
    }

    QPopupMenu *pStyleMenu = new QPopupMenu;
    {
        menuBar()->insertItem("S&tyle", pStyleMenu);

        pStyleMenu->insertItem("&Motif", this, SLOT(motifStyle()) );
        pStyleMenu->insertItem("&CDE", this, SLOT(cdeStyle()) );
        pStyleMenu->insertItem("&Windows", this, SLOT(windowsStyle()) );
        pStyleMenu->insertItem("&Platinum", this, SLOT(platinumStyle()) );
    }

    pControlsMenu = new QPopupMenu;
    {
	menuBar()->insertItem("&Settings", pControlsMenu);

	connect(pControlsMenu, SIGNAL(highlighted(int)),
		this, SLOT(menuInfo(int)));

	MenuBarToggleID = pControlsMenu->insertItem(
	    "&Menu bar", this,
	    SLOT(toggleMenuBar()),
	    CTRL + Key_M);

	// Now accelerators for when the menubar is invisible!

	ToolBarToggleID = pControlsMenu->insertItem(
	    "&Tool bar",
	    this, SLOT(toggleToolBar()),
	    CTRL + Key_T);
	
	StatusBarToggleID = pControlsMenu->insertItem(
	    "&Status bar",
	    this, SLOT(toggleStatusBar()),
	    CTRL + Key_B);
	
	PreviewToggleID = pControlsMenu->insertItem(
	    "&Preview",
	    this, SLOT(togglePreview()),
	    CTRL + Key_P);

	SmoothToggleID = pControlsMenu->insertItem(
	    "Smooth &GUI",
	    this, SLOT(toggleSmooth()),
	    CTRL + Key_G);

	CleanToggleID = pControlsMenu->insertItem(
            "&Full view",
	    this, SLOT(toggleClean()),
	    CTRL + Key_F);

	pControlsMenu->insertSeparator();

	pControlsMenu->insertItem(
	    "&Options...",
	    this, SLOT(showOptions()),
	    CTRL + Key_O);

	pControlsMenu->setCheckable(true);
	pControlsMenu->setItemChecked(MenuBarToggleID, true);
	pControlsMenu->setItemChecked(ToolBarToggleID, true);
	pControlsMenu->setItemChecked(StatusBarToggleID, true);
	pControlsMenu->setItemChecked(PreviewToggleID, true);
	pControlsMenu->setItemChecked(SmoothToggleID, false);
	pControlsMenu->setItemChecked(CleanToggleID, QaConfig::instance()->fullScreen() );
        if ( QaConfig::instance()->smoothGUI() )
            toggleSmooth();
	ShowPreview = true;
    }

    pAccels = new QAccel(this);
    pAccels->connectItem(pAccels->insertItem(CTRL + Key_M),
                         this, SLOT(toggleMenuBar()));
    pAccels->connectItem(pAccels->insertItem(CTRL + Key_B),
                         this, SLOT(toggleStatusBar()));
    pAccels->connectItem(pAccels->insertItem(CTRL + Key_P),
                         this, SLOT(togglePreview()));
    pAccels->connectItem(pAccels->insertItem(CTRL + Key_T),
                         this, SLOT(toggleToolBar()));
    pAccels->connectItem(pAccels->insertItem(CTRL + Key_G),
                         this, SLOT(toggleSmooth()));
    pAccels->connectItem(pAccels->insertItem(CTRL + Key_F),
                         this, SLOT(toggleClean()));
    pAccels->connectItem(pAccels->insertItem(CTRL + Key_O),
                         this, SLOT(showOptions()));

    pAccels->connectItem(pAccels->insertItem(CTRL + Key_N),
                         this, SLOT(newMainWindow()));
    pAccels->connectItem(pAccels->insertItem(CTRL + Key_W),
                         this, SLOT(closeWindow()));
    pAccels->connectItem(pAccels->insertItem(CTRL + Key_Q),
                         this, SLOT(quit()));

    // pWindowMenu is static
    // Make the items "bulleted" so that the active window has a "bullet"
    // next to it.
    
    {
	pWindowMenu = new QPopupMenu;
    }

    // Why won't the window menu appear in any but the first window?
    // Is it because it's allocated in the first window and thus it
    // belongs to that object? But it IS a static variable, so it should
    // belong to ANY object of the class, right?
    //
    // The first time, insertItem() will return 2. All other times, it'll
    // return 0.
    //
    // See Thinking in C++ page 376-378 about copy constructors and static
    // members.
    //
    // There was some posting on the Qt mailinglist that you cannot
    // create static/external Qt objects. Maybe that's why it doesn't work?
    
    menuBar()->insertItem("&Windows", pWindowMenu);
    updateWindowMenu();

#ifdef DEBUG
    pDebugMenu = new QPopupMenu;
    {
	DebugBarID = menuBar()->insertItem("&Debug", pDebugMenu);
        connect( menuBar(), SIGNAL( highlighted( int ) ),
                 this, SLOT( debugSelected( int ) ) );

	DebugID = pDebugMenu->insertItem("Log",
                                         this, SLOT(toggleLog()),
                                         CTRL + Key_L );

	pDebugMenu->setCheckable(true);
	pDebugMenu->setItemChecked(DebugID, true);
    }
#endif

    menuBar()->insertSeparator();

    QPopupMenu *pHelpMenu = new QPopupMenu;
    {
	menuBar()->insertItem("&Help", pHelpMenu);

	pHelpMenu->insertItem("Help...",
			      this, SLOT(showHelp()));
	pHelpMenu->insertSeparator();
	pHelpMenu->insertItem("Supported formats...",
			      this, SLOT(showFormats()));
	pHelpMenu->insertItem("About...",
			      this, SLOT(showAbout()));
	pHelpMenu->insertItem("About Qt...",
			      this, SLOT(showQtInfo()));
    }

    g_pProgress = new QProgressBar(statusBar());
    {
	g_pProgress->setFixedSize(80, 19);
//	g_pProgress->setBackgroundMode(QWidget::PaletteDark);
    }

    g_pLargeInfo = new QLabel(statusBar());
    
    g_pSmallInfo = new QLabel(statusBar());
    {
	g_pSmallInfo->setFixedSize(60, 19);
	g_pSmallInfo->setAlignment(AlignCenter);
    }
    
    statusBar()->addWidget(g_pProgress, 0);
    statusBar()->addWidget(g_pSmallInfo, 0);
    statusBar()->addWidget(g_pLargeInfo, 2);

//     QToolTipGroup *tg = new QToolTipGroup(this);

//     connect(tg, SIGNAL(showTip(const char *)),
// 	    pInfo, SLOT(setText(const char *)));
//     connect(tg, SIGNAL(removeTip()),
// 	    pInfo, SLOT(clear()));
//     QToolTip::add(pBrowser, "Short text", tg, "Looooong text");

    // Note that since the toolbuttons have a border when the
    // mouse button is placed on top of it, the animateClick() function
    // will have no visual effect if the mouse button is not placed
    // on the button as it's only the border that gets recessed.
    //
    // Move the mouse button on top of a toolbutton that has a hotkey,
    // and press that hotkey, and you'll see that it gets recessed.
    
//      pAccels->connectItem(pAccels->insertItem(CTRL + Key_F),
//  			 pForward, SLOT(animateClick()));

//      pAccels->connectItem(pAccels->insertItem(CTRL + Key_U),
//  			 this, SLOT(toggleFullScreen()));

    // We want the browser first.
    setView( false );

    resize( QaConfig::instance()->windowSize() );
    setHotList( QaConfig::instance()->hotList(),
                QaConfig::instance()->nameHotList() );
    QString s;
    s.sprintf("Welcome to %s.", APP_NAME);
    statusBar()->message((const char *)s, 1000 * 5);
}


/*!
  Destroys the window.
*/

QmMainWindow::~QmMainWindow()
{
    if ( pOptions != 0 )
        delete pOptions;
}


/*!
  Update the window menu to show all windows.
*/

/*
  The menu variable is static and is thusly shared by all objects of this
  class.
*/

void
QmMainWindow::updateWindowMenu(void)
{
    pWindowMenu->clear();
    
    QListIterator<QmMainWindow> wit(WindowList);

    // Build the menu

    for(QmMainWindow *w = wit.toFirst(); w != 0; w = ++wit)
    {
	// Place the mouse over the new window, maybe? Can be done
	// like this:
	// QCursor::setPos(100, 100);

	pWindowMenu->insertItem(w->caption(), w, SLOT(raise()));
    }
}


/*!
  Toggles between the image view and the file browser.
*/

void
QmMainWindow::toggleView()
{
    setView( pStack->visibleWidget() == pBrowser );
}

/*!
  Toggles between the image view and the file browser.
*/

void
QmMainWindow::setView( bool image )
{
    if ( image )
    {
	// Disable/enable toolbuttons appropriate to this view

  	pBack->setEnabled(true);
  	pForward->setEnabled(true);
	pSlide->setEnabled(true);
	pFileOpen->setOn(true);
  	pZoomOut->setEnabled(true);
  	pZoomIn->setEnabled(true);

        if ( pControlsMenu->isItemChecked( CleanToggleID ) )
            setClean( true );

        // Save the focus for later retrieval
        pOldFocus = qApp->focusWidget();

	pStack->raiseWidget(g_pImageViewBrowser);
	g_pImageViewBrowser->imageView()->setFocus();
        if ( IsFullScreen )
            g_pImageViewBrowser->imageView()->grabKeyboard();
    }
    else
    {
	// Disable/enable toolbuttons appropriate to this view

  	pBack->setEnabled(false);
  	pForward->setEnabled(false);
	pSlide->setEnabled(false);
	pFileOpen->setOn(false);
	pZoomOut->setEnabled(false);
	pZoomIn->setEnabled(false);

        if ( IsFullScreen )
            g_pImageViewBrowser->imageView()->releaseKeyboard();
        setClean( false );

	pStack->raiseWidget(pBrowser);
        if ( pOldFocus != 0 )
            pOldFocus->setFocus();
    }
}


/*!
  Starts/stops slideshow.
*/

void
QmMainWindow::toggleSlideshow()
{
    CHECK_PTR(pSlideshowTimer);

    static bool SlideshowRunning = false;

    if(pSlideshowTimer->isActive())
	pSlideshowTimer->stop();
    
    if(SlideshowRunning)
	SlideshowRunning = false;
    else
    {
	pSlideshowTimer->start(SlideshowWait);
	SlideshowRunning = true;
    }
}


/*!

*/

void
QmMainWindow::zoomUp()
{
    CHECK_PTR(pZoomOut);
    CHECK_PTR(pZoomIn);

    int Zoom = g_pImageViewBrowser->imageView()->currentZoom() + ZoomStep;

//    pZoomIn->setEnabled(true);

    if(Zoom > 200)
	return;
//	pZoomOut->setEnabled(false); // Crashes!! Why?

    g_pImageViewBrowser->imageView()->zoom(Zoom);
}


/*!

*/

void
QmMainWindow::zoomDown()
{
    CHECK_PTR(pZoomOut);
    CHECK_PTR(pZoomIn);

    int Zoom = g_pImageViewBrowser->imageView()->currentZoom() - ZoomStep;

//    pZoomOut->setEnabled(true);

    if(Zoom < 20)
	return;
//	pZoomIn->setEnabled(false);

    g_pImageViewBrowser->imageView()->zoom(Zoom);
}


/*!

*/

void
QmMainWindow::zoomEntered(
    const char *ZoomFactor)
{
//      CHECK_PTR(pImageView);

    QString s(ZoomFactor);
    g_pImageViewBrowser->imageView()->zoom(s.toInt());
}


/*!
  Shows the next image.
*/

void
QmMainWindow::nextImage()
{
    if( pStack->visibleWidget() == g_pImageViewBrowser->imageView() )
	pBrowser->queryNextImage();
}


/*!
  Shows the previous image.
*/

void
QmMainWindow::prevImage()
{
    if(pStack->visibleWidget() == g_pImageViewBrowser->imageView())
	pBrowser->queryPrevImage();
}


/*!
  Loads the image specified by \a Filename. The image will be show in the
  image view.

  \sa showImage(const QPixmap &)
*/

void
QmMainWindow::showImage(
    const char *Filename)
{
    qDebug( "QmMainWindow::showImage( \"%s\" )", Filename );
    g_pSmallInfo->clear();

    QFileInfo fi(Filename);
    QString s;
    s.sprintf("Loading image %s...", (const char *)fi.fileName());
    g_pLargeInfo->setText((const char *)s);

    QApplication::setOverrideCursor(waitCursor);

    bool ImageLoaded = g_pImageViewBrowser->loadImage(Filename);

    qDebug( "Setting focus to QmImageView" );
//      pStack->raiseWidget(g_pImageViewBrowser);
//      g_pImageViewBrowser->imageView()->setFocus();
    if( pStack->visibleWidget() == pBrowser )
        toggleView();

    QApplication::restoreOverrideCursor();

    if( ! ImageLoaded )
    {
	s.sprintf("Failed to load the image %s", (const char *)fi.fileName());
	statusBar()->message((const char *)s, 1000 * 5);
    }
    g_pImageViewBrowser->imageView()->setFocus();
}


/*!
  Shows the pixmap specified by \a Pixmap in the image view.

  \sa showImage(const char *Filename)
*/

void
QmMainWindow::showImage(
    const QPixmap &Pixmap)
{
//      pStack->raiseWidget(pImageView);
//      pImageView->setFocus();

    QApplication::setOverrideCursor(waitCursor);
    g_pImageViewBrowser->imageView()->setPixmap(Pixmap);
//      if( pStack->visibleWidget() == pBrowser )
    setView( true );
    QApplication::restoreOverrideCursor();
}


/*!
  Toggles the menu bar on/off.
*/

void
QmMainWindow::toggleMenuBar()
{
    if ( menuBar()->isVisible() )
    {
        menuBar()->hide();
        pControlsMenu->setItemChecked(MenuBarToggleID, false);
    }
    else
    {
        menuBar()->show();
        pControlsMenu->setItemChecked(MenuBarToggleID, true);
    }
    if ( IsFullScreen )
        FullMenuBarHidden = !menuBar()->isVisible();
    else
        MenuBarHidden = !menuBar()->isVisible();
}


/*!
  Toggles the toolbar on/off.
*/

void
QmMainWindow::toggleToolBar()
{
    if(pToolBar->isVisible())
    {
        pToolBar->hide();
        pControlsMenu->setItemChecked(ToolBarToggleID, false);
    }
    else
    {
        pToolBar->show();
        pControlsMenu->setItemChecked(ToolBarToggleID, true);
    }
    if ( IsFullScreen )
        FullToolBarHidden = !pToolBar->isVisible();
    else
        ToolBarHidden = !pToolBar->isVisible();
}


/*!
  Toggles the status bar on/off.
*/

void
QmMainWindow::toggleStatusBar()
{
    if(statusBar()->isVisible())
    {
        statusBar()->hide();
        pControlsMenu->setItemChecked(StatusBarToggleID, false);
    }
    else
    {
        statusBar()->show();
        pControlsMenu->setItemChecked(StatusBarToggleID, true);
    }
    if ( IsFullScreen )
        FullStatusBarHidden = !statusBar()->isVisible();
    else
        StatusBarHidden = !statusBar()->isVisible();
}


/*!
  Toggles the preview on/off.
*/

void
QmMainWindow::togglePreview()
{
    if(ShowPreview == true)
    {
	pBrowser->setPreview(false);
        pControlsMenu->setItemChecked(PreviewToggleID, false);
	ShowPreview = false;
    }
    else
    {
	pBrowser->setPreview(true);
        pControlsMenu->setItemChecked(PreviewToggleID, true);
	ShowPreview = true;
    }
}


/*!
  Toggle between opaque modes on the GUI.
  Currently only the splitters are affected.
*/

void QmMainWindow::toggleSmooth()
{
    QSplitter *s1 = pBrowser->verticalSplitter();
    QSplitter *s2 = pBrowser->horizontalSplitter();
    s1->setOpaqueResize( !s1->opaqueResize() );
    s2->setOpaqueResize( !s2->opaqueResize() );
    pControlsMenu->setItemChecked( SmoothToggleID, s1->opaqueResize() );

    QaConfig::instance()->setSmoothGUI( s1->opaqueResize() );
}

/*!
  Toggles full screen mode.
*/

void
QmMainWindow::toggleFullScreen()
{
    static bool		FullScreen		= false;
    static bool		ToolBarVisible		= pToolBar->isVisible();
    static bool		StatusBarVisible	= statusBar()->isVisible();
    static bool		MenuBarVisible		= menuBar()->isVisible();
    static QPoint	OldPosition		= pos();
    static QSize	OldSize			= size();

    if( ! FullScreen)
    {
	// Save current settings
	
	OldPosition		= pos();
	OldSize			= size();
	ToolBarVisible		= pToolBar->isVisible();
	StatusBarVisible	= statusBar()->isVisible();
	MenuBarVisible		= menuBar()->isVisible();

	hide();
	pToolBar->hide();
	statusBar()->hide();
	menuBar()->hide();

	// Get the desktop widget to examine the screen width and height
	
	QWidget *d = QApplication::desktop();
	resize(d->width(), d->height());

	// Damn! I can't get the window to only have a border around,
	// but no title bar. It doesn't work by clearing the WStyle_Title
	// bit (clearing the bit work, but Qt doesn't care).
	
//   	WFlags f = WStyle_Customize | WStyle_DialogBorder;
//   	f &= ~WStyle_Title; // Clear the WStyle_Title bit

//  	recreate(parentWidget(), f, QPoint(0, 0), true);
	FullScreen = true;
    }
    else
    {
	// Lets hide the window. It gets less "painful" for the user to
	// see all the resizing, adding of toolbars etc. ;-)
	
	hide();

	// Load last saved settings
	
	if(ToolBarVisible)
	    pToolBar->show();

	if(StatusBarVisible)
	    statusBar()->show();

	if(MenuBarVisible)
	    menuBar()->show();
	
	resize(OldSize);
	
	reparent(parentWidget(),
		 WStyle_Customize | WStyle_NormalBorder,
		 OldPosition, true);
	
	FullScreen = false;
    }
}


/*!
  Shows long description of a highlighted menu item in the status bar.
*/

void
QmMainWindow::menuInfo(
    int MenuID)
{
    if(MenuID == MenuBarToggleID)
	g_pLargeInfo->setText("Toggles the menubar on/off.");
    else if(MenuID == PreviewToggleID)
	g_pLargeInfo->setText("Toggles the preview on/off.");
    else if(MenuID == StatusBarToggleID)
	g_pLargeInfo->setText("Toggles the statusbar on/off.");
    else if(MenuID == ToolBarToggleID)
	g_pLargeInfo->setText("Toggles the toolbar on/off.");
    else
	g_pLargeInfo->setText("");
}


/*!
  Shows the Options dialog.
*/

void
QmMainWindow::showOptions()
{
//      QString s;
//      s.sprintf( "QmMainWindow::showOptions(): Not implemented yet.\n" );
//      QMessageBox::information(this, "Options", (const char *)s);
    if ( pOptions == 0 )
    {
        pOptions = new QaOptions;
//          pOptions = new QVBox( 0, "opts" );
//          {
//              QHBox *hbox = new QHBox( pOptions, "opts-hbox" );
//              {
//                  QVGroupBox *opts = new QVGroupBox( QString( "Save" ), hbox, "options-save" );
//                  {
//                      QLabel *info = new QLabel( "Select the options you wish to save when closing QtVu", opts, "opts-info" );
//                      info->setAlignment( Qt::AlignLeft | Qt::WordBreak );
//                      QFrame *line = new QFrame( opts, "opts-split" );
//                      line->setFrameStyle( QFrame::HLine | QFrame::Raised );

//                      QCheckBox *menu = new QCheckBox( QString( "Menubar" ), opts, "opts-menu" );
//                      menu->setChecked( true );
//                      QCheckBox *preview = new QCheckBox( QString( "Preview" ), opts, "opts-preview" );
//                      preview->setChecked( true );
//                      QCheckBox *status = new QCheckBox( QString( "StatusBar" ), opts, "opts-status" );
//                      status->setChecked( true );
//                      QCheckBox *toolbar = new QCheckBox( QString( "ToolBar" ), opts, "opts-tool" );
//                      toolbar->setChecked( true );
//                  }

//                  QVGroupBox *scroll = new QVGroupBox( QString( "Scrolling" ), hbox, "options-scroll" );
//                  {
//                      QSlider *scrolldelay = new QSlider( 3, 40,
//                                                          4, QaConfig::instance()->scrollDelay(),
//                                                          QSlider::Horizontal,
//                                                          scroll, "opts-scrolldelay" );
//                      scrolldelay->setMinimumSize( 100, 10 );
//                      scrolldelay->setTickmarks( QSlider::Above );
//                      QSlider *scrollpixels = new QSlider( 1, 6,
//                                                           1, QaConfig::instance()->scrollPixels(),
//                                                           QSlider::Horizontal,
//                                                           scroll, "opts-scrollpixels" );
//                      scrollpixels->setMinimumSize( 100, 10 );
//                      scrollpixels->setTickmarks( QSlider::Above );
//                  }
//              }
//              hbox->setSpacing( 3 );

//              QHBox *buttons = new QHBox( pOptions, "opts-buttons" );
//              {
//                  (void)new QPushButton( "&Ok", buttons, "button-ok" );
//                  (void)new QPushButton( "&Apply", buttons, "button-apply" );
//                  (void)new QPushButton( "&Cancel", buttons, "button-cancel" );
//              }
//              buttons->setSpacing( 3 );
//          }
//          pOptions->setCaption( "QtVu - Options" );
//          pOptions->show();
//          pOptions->move( pos() + QPoint( (width() - pOptions->width())/2, (height() - pOptions->height())/2 ) );

//          QAccel *pOptionAccels = new QAccel( pOptions );
//          pOptionAccels->connectItem(pOptionAccels->insertItem(CTRL + Key_Q),
//                                     this, SLOT(quit()));
    }
//          pOptions->setActiveWindow();
    pOptions->show();
    pOptions->raise();
}


/*!
  Shows the formats supported.

  The formats supported depends on the compilation of the QImageIO
  extension. It can be compiled to support various formats.
*/

void
QmMainWindow::showFormats()
{
    static QmListDialog *pDialog = new QmListDialog;

    pDialog->show();
}


/*!
  Shows the Help system.
*/

void
QmMainWindow::showHelp()
{
    QString s;
    s.sprintf( "QmMainWindow::showHelp(): Not implemented yet.\n" );
    QMessageBox::information(this, "Help", (const char *)s);
}


/*!
  Shows some short information about the program.
*/

void
QmMainWindow::showAbout()
{
    QString s;
    s.sprintf(
	"%s %s\n"
	"\n"
	"This application is distributed under the\n"
	"GNU General Public License (GPL), version 2 or later,\n"
	"as defined by the Free Software Foundation.\n"
	"\n"
	"\n"
	"(C) 1998-1999 Marius Sundbakken <marius@fix.no>\n"
        "            & Jan Borsodi <amos@abn.hibu.no>\n",
	APP_NAME,
	APP_VERSION);
    
    QMessageBox::about(this, APP_NAME, (const char *)s);
}


/*!
  Shows the standard Qt information dialog box.
*/

void
QmMainWindow::showQtInfo()
{
    QMessageBox::aboutQt(this, APP_NAME);
}


/*!
  Opens a new main window.
*/

void
QmMainWindow::newMainWindow()
{
    QmMainWindow *pNewWindow = new QmMainWindow;
    pNewWindow->show();
}


/*!
  Closes the window.
*/

void
QmMainWindow::closeWindow()
{
    if(WindowList.count() == 1)
    {
                saveState();
//  		delete this;
		qApp->quit();
    }
    else
    {
		WindowList.removeRef(this);
		delete this;
    }

    updateWindowMenu();
}


/*!
  Deletes the file \a FileToDelete if the user wants to.
*/

/*
  Write some replacement for QMessageBox::warning() which disables the Delete button
  for half a second or so to get rid of "multiple return key press out of habit" problems.
*/

bool
QmMainWindow::deleteItem(
    const char *FileToDelete)
{
    QFileInfo ItemToDelete(FileToDelete);

    return deleteItem(ItemToDelete);
}

/*!
  Deletes the file/dir if the user want so. Returns true if the file was
  deleted, false otherwise.
 */

bool
QmMainWindow::deleteItem(
    const QFileInfo &ItemToDelete)
{
    QString s;
	    
    if(ItemToDelete.isDir())
    {
	s.sprintf("-- NOT IMPLEMENTED YET --\n"
		  "Are you sure you want to DELETE the directory\n"
		  "%s\n"
		  "and all of its contents?",
		  ItemToDelete.filePath().ascii());
    }
    else
    {
	cout << "File path: " << ItemToDelete.filePath() << endl
	     << "File name: " << (const char *)ItemToDelete.fileName() << endl;

	s.sprintf("Are you sure you want to DELETE the file\n"
		  "%s?",
		  (const char *)ItemToDelete.fileName());

	int rc = QMessageBox::warning(0, APP_NAME,
				      (const char *)s,
				      "Delete", "Cancel", 0, 1, 1);

	if(rc == 0)
	{
	    // Delete file

	    if( ! QFile::remove(ItemToDelete.filePath()))
	    {
		s.sprintf("Failed to delete the file\n"
			  "%s",
			  ItemToDelete.filePath().ascii());

		QMessageBox::warning(0, APP_NAME,
				     (const char *)s);

		return false;
	    }
	}
    }

    return true;
    
//    cout << "QmMainWindow::deleteItem(): rc = " << rc << endl;
}


void QmMainWindow::setPictureFile( QString fname )
{
	showImage( (const char *)fname );
}

void QmMainWindow::setPath( QString dir )
{
	pBrowser->setPath( dir );
}

QString QmMainWindow::getIconPath( QString name )
{
	QFileInfo f;
	f.setFile( QDir::homeDirPath() + "/." + QString( APP_NAME ) + "/icons/" + name );
	if ( f.exists() )
		return f.absFilePath();
	f.setFile( "/usr/share/icons/" + name );
	if ( f.exists() )
		return f.absFilePath();
	f.setFile( name );
	if ( f.exists() )
		return f.absFilePath();
	return QString();
}

QString QmMainWindow::configFile()
{
	return QString( QDir::homeDirPath() + "/.QtVu/config" );
}

QString QmMainWindow::oldConfigFile()
{
	return QString( QDir::homeDirPath() + "/.qtvurc" );
}

QStringList QmMainWindow::hotList()
{
    QmHotList *hot = pBrowser->hotList();
    QListViewItem *first = hot->firstChild();
    QStringList s;
    while ( first )
    {
        s.append( first->text( 1 ) );
        first = first->itemBelow();
    }
    return s;
}

QStringList QmMainWindow::nameHotList()
{
    QmHotList *hot = pBrowser->hotList();
    QListViewItem *first = hot->firstChild();
    QStringList s;
    while ( first )
    {
        s.append( first->text( 0 ) );
        first = first->itemBelow();
    }
    return s;
}


void QmMainWindow::setHotList( const QStringList &l, const QStringList &nl )
{
    QmHotList *hot = pBrowser->hotList();
    hot->clear();
    QStringList::ConstIterator i2 = nl.begin();
    for ( QStringList::ConstIterator i = l.begin(); i != l.end() && i2 != nl.end(); ++i, ++i2 )
    {
        QString nstr( (*i2) );
        QString str( (*i) );
        (void) new QmListViewItem( hot,
                                   nstr,
                                   str);
    }
}

void QmMainWindow::motifStyle()
{
    qApp->setStyle( new QMotifStyle );
}

void QmMainWindow::cdeStyle()
{
    qApp->setStyle( new QCDEStyle );
}

void QmMainWindow::windowsStyle()
{
    qApp->setStyle( new QWindowsStyle );
}

void QmMainWindow::platinumStyle()
{
    qApp->setStyle( new QPlatinumStyle );
}

void QmMainWindow::show()
{
    QMainWindow::show();
    QTabWidget *tab = pBrowser->tabBar();
    if ( QaConfig::instance()->hotDir() )
        tab->showPage( pBrowser->hotList() );
}

void QmMainWindow::quit()
{
    saveState();
    qApp->quit();
}

void QmMainWindow::saveState()
{
    QaConfig::instance()->setWindowSize( size() );

#ifdef DEBUG
    QaConfig::instance()->setLogSize( pLog->size() );
#endif

    QaConfig::instance()->setHotList( hotList() );
    QaConfig::instance()->setNameHotList( nameHotList() );
    QaConfig::instance()->setHotDir( dynamic_cast<QmHotList *>(pBrowser->tabBar()->currentPage()) != 0 );
    QaConfig::instance()->setSplitterSizes( pBrowser->verticalSplitter()->sizes() );
    QaConfig::instance()->setDividerSizes( pBrowser->horizontalSplitter()->sizes() );
    QaConfig::instance()->setSmoothGUI( pBrowser->verticalSplitter()->opaqueResize() );
}

#ifdef DEBUG
void QmMainWindow::toggleLog()
{
    CHECK_PTR( pLog );
    if ( pLog->isVisible() )
    {
        pLog->hide();
	pDebugMenu->setItemChecked(DebugID, false);
    }
    else
    {        
//          pLog->setActiveWindow();
        pLog->show();
        pLog->raise();
	pDebugMenu->setItemChecked(DebugID, true);
    }
}
#endif

void QmMainWindow::toggleClean()
{
    pControlsMenu->setItemChecked( CleanToggleID, !pControlsMenu->isItemChecked( CleanToggleID ) );
    QaConfig::instance()->setFullScreen( pControlsMenu->isItemChecked( CleanToggleID ) );
    if ( pStack->visibleWidget() != pBrowser )
    {
        setClean( pControlsMenu->isItemChecked( CleanToggleID ) );
    }
}

void QmMainWindow::setClean( bool clean )
{
    if ( clean == IsFullScreen )
        return;

    IsFullScreen = clean;
    if ( clean )
    {
        if ( FullMenuBarHidden )
            menuBar()->hide();
        else
            menuBar()->show();
         if ( FullToolBarHidden )
            pToolBar->hide();
        else
            pToolBar->show();
        if ( FullStatusBarHidden )
            statusBar()->hide();
        else
            statusBar()->show();
        OldGeo = geometry();
//          clearWFlags( getWFlags() );
//          setWFlags( WStyle_Customize | WStyle_NoBorder );
//          move( 0, 0 );
//          resize( QApplication::desktop()->size() );
//          g_pImageViewBrowser->reparent( 0,  WStyle_Customize | WStyle_NoBorder | WStyle_StaysOnTop, QPoint( 0, 0 ) );
//          g_pImageViewBrowser->resize( QApplication::desktop()->size() );
//          g_pImageViewBrowser->show();
//          g_pImageViewBrowser->imageView()->setFocus();
//          g_pImageViewBrowser->imageView()->grabKeyboard();

        reparent( 0,  WStyle_Customize | WStyle_NoBorder | WStyle_StaysOnTop, QPoint( 0, 0 ) );
        resize( QApplication::desktop()->size() );
        show();
//        g_pImageViewBrowser->imageView()->grabKeyboard();

//          setAutoMask( true );
//          QRegion reg( QRect( QPoint( 0, 0 ), size() ),
//                       QRegion::Rectangle );
//          setMask( reg );
//        move( 0, 0 );
//        resize( QApplication::desktop()->size() );
//        raise();
    }
    else
    {
        if ( MenuBarHidden )
            menuBar()->hide();
        else
            menuBar()->show();
        if ( ToolBarHidden )
            pToolBar->hide();
        else
            pToolBar->show();
        if ( StatusBarHidden )
            statusBar()->hide();
        else
            statusBar()->show();
//          setWFlags( WStyle_Customize | WStyle_NoBorder );
//          setGeometry( OldGeo );
//          g_pImageViewBrowser->reparent( pStack, 0, QPoint( 0, 0 ) );
//          g_pImageViewBrowser->show();

        reparent( 0, 0, QPoint( 0, 0 ) );
        resize( OldGeo.size() );
        move( OldGeo.topLeft() );
        show();

//          g_pImageViewBrowser->imageView()->releaseKeyboard();
//          setAutoMask( false );
//          clearMask();
    }
}

#ifdef DEBUG
void QmMainWindow::debugSelected( int i )
{
    if ( i == DebugBarID )
    {
        pDebugMenu->setItemChecked( DebugID, pLog->isVisible() );
    }
}
#endif

void QmMainWindow::resizeEvent( QResizeEvent *e )
{
    QMainWindow::resizeEvent( e );
    if ( IsFullScreen )
    {
        if ( autoMask() )
            updateMask();
    }
}

void QmMainWindow::moveEvent( QMoveEvent *e )
{
    QMainWindow::moveEvent( e );
    if ( IsFullScreen )
    {
        if ( autoMask() )
            updateMask();
    }
}

void QmMainWindow::updateMask()
{
    QRegion reg( QRect( QPoint( 0, 0 ), size() ) );
    setMask( reg );
}
