/* ;-*-c++-*-
 *
 * qmmainwindow.h
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <30-Jul-1999 03:42:46 amos>
 */

#ifndef QMMAINWINDOW_H_INCLUDED
#define QMMAINWINDOW_H_INCLUDED

#include "qmimageview.h"
#include "qmimageviewbrowser.h"
#include "qmbrowser.h"
#include "qaoptions.hpp"

#include <qfileinfo.h>
#include <qframe.h>
#include <qimage.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qmainwindow.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qscrollview.h>
#include <qstatusbar.h>
#include <qtimer.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qwidgetstack.h>
#include <qstring.h>
#include <qcombobox.h>
#include <qlist.h>
#include <qwidget.h>
#include <qdir.h>
#include <qtextview.h>
#include <qvbox.h>
#include <qtextview.h>
#include <qrect.h>
#include <qevent.h>

class QmMainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    QmMainWindow(QWidget *Parent = 0,
		 const char *Name = 0);
    virtual ~QmMainWindow();

    static bool	deleteItem(const QFileInfo &ItemToDelete);
    static bool	deleteItem(const char *FileToDelete);
    void setPictureFile( QString fname );
    void setPath( QString dir );
    QString getIconPath( QString name );
    static QString configFile();
    static QString oldConfigFile();
    QStringList hotList();
    QStringList nameHotList();
    void setHotList( const QStringList &l, const QStringList &nl );
    void saveState();
    QTextView *logger() { return pLog; }

    virtual void show();

protected:
    virtual void resizeEvent( QResizeEvent *e );
    virtual void moveEvent( QMoveEvent *e );
    virtual void updateMask();

private:
    void		updateWindowMenu(void);

    QComboBox		*pZoom;
    QFrame		*pImage;
    QImage		TheImage;
    QLabel		*pInfo;
    QPopupMenu		*pControlsMenu;
    QTimer		*pSlideshowTimer;
    QToolBar		*pToolBar;
    QToolButton		*pFileOpen;
    QToolButton		*pSlide;
    QToolButton		*pZoomIn;
    QToolButton		*pZoomOut;
    QToolButton		*pBack;
    QToolButton		*pForward;
    QWidgetStack	*pStack;
    QmBrowser		*pBrowser;
//      QmImageView		*pImageView;
    QaOptions		*pOptions;
    QWidget		*pOldFocus;
    QRect		OldGeo;

    bool		IsFullScreen;

    bool		FullMenuBarHidden;
    bool		FullToolBarHidden;
    bool		FullStatusBarHidden;

    bool		MenuBarHidden;
    bool		ToolBarHidden;
    bool		StatusBarHidden;

    static QPopupMenu	*pWindowMenu;
    static QList<QmMainWindow>	WindowList;
    
    int			MenuBarToggleID;
    int			PreviewToggleID;
    int			StatusBarToggleID;
    int			ToolBarToggleID;
    int			SmoothToggleID;
    int			CleanToggleID;

    bool		ShowPreview;

#ifdef DEBUG    
    QPopupMenu		*pDebugMenu;
    QTextView		*pLog;
    int			DebugID;
    int			DebugBarID;
#endif

    static const int	ImageViewID	= 1;
    static const int	BrowserID	= 2;
    static const int	ZoomStep	= 20;
    static const int	SlideshowWait	= 5000;

public slots:
    void quit();

private slots:
    void closeWindow();
    void menuInfo(int MenuID);
    void newMainWindow();
    void nextImage();
    void prevImage();

    void showAbout();
    void showFormats();
    void showHelp();
    void showImage(const QPixmap &Pixmap);
    void showImage(const char *Filename);
    void showOptions();
    void showQtInfo();

    void toggleFullScreen();
    void toggleMenuBar();
    void togglePreview();
    void toggleSlideshow();
    void toggleStatusBar();
    void toggleToolBar();
    void toggleView();
    void toggleSmooth();
    void toggleClean();
    void setView( bool image );
    void setClean( bool clean );

    void zoomDown();
    void zoomEntered(const char *ZoomFactor);
    void zoomUp();

    void motifStyle();
    void cdeStyle();
    void windowsStyle();
    void platinumStyle();
    void toggleLog();
#ifdef DEBUG
    void debugSelected( int );
#endif
};

void myMessageOutput( QtMsgType type, const char *msg );

#endif // QMMAINWINDOW_H_INCLUDED
